//view的公共基类，提供的能力
//  依据配置生成界面内容
//  view嵌套，创建、加载、刷新下级view
//  启动页面内容的刷新
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/Deferred",
    "../widget/_WidgetBase",
    "../config"
],function(declare,lang,Deferred,_WidgetBase,config){
    // module:
    //      cq/app/view/_ViewBase
    
    return declare("",_WidgetBase,{
        // summary:
        //      提供公共的view加载、切换、销毁相关的接口和基本实现
        
        // viewContainerNode:domNode
        //      用于存放下级view的容器节点，在填充本view之后才被赋值
        // tags:
        //      private
        viewContainerNode:null,
        
        // subViews:object
        //      用户存放下级view的参数，view实例化时由cq/app/config初始化
        //      没有下级view时可以不设置
        subViews:null,
        
        // defaultView:string
        //      默认显示的下级view的viewId
        //      由下级view时必须设置，否则可能会出错
        defaultView:"",
        
        showView:function(road,params){
            // summary:
            //      依据view路径显示view
            //      显示自己，若由下级view，(创建)显示下级view
            //      触发对象变化的事件，刷新内容
            //      异步函数，所有的下级view都ready之后，有跟view执行resize方法，重算所有子节点的大小
            this.show();
            var callback = null,viewParam = {
                road:road,
                params:params
            };
            var def = new Deferred();
            if(this.subViews){
                if(0===road.length){
                    road.push(this.defaultView);
                }
                
                var newRoad = [],viewId = road[0];
                for(var index=1;index<road.length;index++){
                    newRoad.push(road[index]);
                }
                this.onCqChanged(this,viewParam);
                if(!this.subViews[viewId].view){
                    config.createView(this.subViews[viewId],viewId,this.viewContainerNode).then(lang.hitch(this,function(view){
                        this.subViews[viewId].view = view;
                        this.subViews[viewId].view.showView(newRoad,params).then(function(){
                            def.resolve();
                        });
                    }));
                }else{
                    this.subViews[viewId].view.showView(newRoad,params).then(function(){
                        def.resolve();
                    });
                }
            }else{
                this.onCqChanged(this,viewParam);
                window.free();
                def.resolve();
            }
            return def;
        }
    });
});
