﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-attr",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dijit/registry",
    "./_WidgetBase",
    "./utils/listLayout"
],function(declare,lang,array,domAttr,domConstruct,domStyle,registry,_CqWidgetBase,listLayout){
    // module:
    //      cq/app/widget/_ListMixin
    
    //对象属性
    //  dataProvider:数据提供者
    //  targetMethod:获取数据url的方法
    //  itemPropMethod:数组准备好后依据数据生成列表项参数的方法名称
    //  queryParamMethod:查询参数方法
    //  itemType:列表项类型
    //  workMode:工作模式
    //  pageSize:分页行数，所有的动态数据列表都要分页
    //  clickMethod:点击事件方法名
    
    //列表支持layout、layout标题和没有数据时隐藏
    //  layoutMethod:读取列表结构的方法
    //  hideWhenNoData:没有数据时隐藏
    
    var props = "label:'{label}',hideWhenNoData:'{hideWhenNoData}',layoutMethod:'{layoutMethod}',dataProvider:'{dataProvider}',targetMethod:'{targetMethod}',itemPropMethod:'{itemPropMethod}',queryParamMethod:'{queryParamMethod}',itemType:'{itemType}',workMode:'{workMode}',pageSize:'{pageSize}',showPager:'{showPager}',clickMethod:'{clickMethod}'";
    //列表对象，支持选择
    return declare("",[_CqWidgetBase],{
        // summary:
        //      提供一个可配置的列表
        //      实现列表通常要处理的问题，数据分页、加载、显示、刷新、销毁等
        
        // htmlTemp:string
        //      本对象的html模版，每个通过layout配置显示在界面上的widget都要实现本属性
        htmlTemp:'<div id="{id}" class="cq-list {listClass}" data-dojo-type="cq/app/widget/web/List" allData=\'{j:allData}\' items=\'{j:items}\' selected="{selected}" data-dojo-props="'+props+'"></div>',
        
        // items:Array
        //      生成列表的原始数据，没有提供数据源时，通过本属性指定列表的固定内容
        items:null,
        
        selected:"",
        
        // label:string
        //      显示在所有列表项之前的提示信息，可以不设置
        label:"",
        
        // allData:object
        //      显示动态获取的备选项之前的固定备选项数据
        allData:null,
        
        // dataProvider:string
        //      提供操作的对象的模块id
        dataProvider:"",
        
        // targetMethod:string
        //      dataProvider提供的获取读取数据地址的方法的方法名
        targetMethod:"",
        
        // itemPropMethod:string
        //      dataProvider提供的生成获取列表项的参数的方法的方法名
        itemPropMethod:"",
        
        // queryParamMethod:string
        //      dataProvider提供的依据cqRefInfo生成查询条件的方法的方法名
        queryParamMethod:"",
        
        // clickMethod:string
        //      dataProvider提供的处理列表项点击事件方法的方法名
        clickMethod:"",
        
        // itemType:string
        //      依据数据生成的列表项的类型
        //      需要在item目录下实现对应的模块
        itemType:"",
        
        // workMode:string
        //      工作模式,
        //          "single":单选模式
        //          "multi":多选模式
        //          "action":点击响应
        workMode:"",
        
        layoutMethod:"",
        hideWhenNoData:false,
        
        //分页页面大小
        pageSize:20,
        
        //当前页序
        start:0,
        
        //列表项原型对象
        // tags:
        //      private
        _itemRender:null,
        
        // refreshOnReady:boolean
        //      创建之后是否要指定执行一次刷新
        refreshOnReady:false,
        
        createLabelNode:function(){
            // summary:
            //      创建标签节点，部分列表不需要有这个节点，所有提供了接口
            this.labelNode = domConstruct.create("div",{
                innerHTML:this.label
            },this.containerNode,"before");
        },
        //执行初始化
        buildRendering:function(){
            this.inherited(arguments);
            if(this.label){
                this.createLabelNode();
            }
            this.initPageNode();
            this.pageSize = this.pageSize-0;
            this.pageSize = this.pageSize||20;
            if(this.items&&lang.isString(this.items)){
                this.items = JSON.parse(this.items);
            }
            this.hideWhenNoData = true===this.hideWhenNoData||"true"===this.hideWhenNoData;
            this.selectedItems = [];
            if(this.dataProvider){
                //有数据提供者，初始化数据提供者
                if(lang.isString(this.dataProvider)){
                    require([this.dataProvider],lang.hitch(this,this._onDataProviderReady));
                }else{
                    this._onDataProviderReady(this.dataProvider);
                }
            }else{
                this._initItemRender();
            }
        },
        initPageNode:function(){
            // summary:
            //      初始化分页区域节点
        },
        _onDataProviderReady:function(dataProvider){
            // summary:
            //      加载完数据提供者之后的处理
            // tags:
            //      private
            this.dataProvider = dataProvider;
            this._initItemRender();
        },
        getItemTypeId:function(itemType){
            return "cq/app/widget/web/item/"+itemType;
        },
        _initItemRender:function(){
            // summary:
            //      加载列表项对应模块，加载完成之后，初始化列表
            if(this.itemType){
                require([this.getItemTypeId(this.itemType)],lang.hitch(this,this._onItemRenderReady));
            }else{
                this._onItemRenderReady(this.itemType);
            }
        },
        _onItemRenderReady:function(itemRender){
            this._itemRender = itemRender;
            if(this.dataProvider&&this.targetMethod){
                if(this.layoutMethod){
                    this.layout = listLayout.initLayoutFormatter(this.dataProvider[this.layoutMethod]());
                    this.fieldItem = new this._itemRender({
                        layout:this.layout
                    });
                }
                //使用store
                require(["dojo/store/JsonRest"],lang.hitch(this,function(JsonRest){
                    this.store = new JsonRest({
                        target:this.dataProvider[this.targetMethod]()
                    });
                    if(this.refreshOnReady){
                        this.refresh();
                    }
                }));
            }else{
                this.onDataReady(this.items);
            }
        },
        //刷新界面，从第一页开始显示
        refresh:function(){
            // summary:
            //      安全的刷新列表的接口
            if(!this.canReload()){
                return;
            }
            this.initList();
            if(this.dataProvider){
                if(this.store){
                    this.loadMore();
                }else{
                    this.refreshOnReady = true;
                }
            }else{
                this.onDataReady(this.items);
            }
        },
        _lastQueryStart:-1,
        _lastQueryTime:0,
        _lastQueryParam:"",
        loadMore:function(index){
            if(index>=0){
                this.start = this.pageSize*index;
            }
            this._lastQueryStart = this.start;
            this._lastQueryParam = JSON.stringify(this.queryParam);
            this._lastQueryTime = new Date();
            
            if(this.queryResult&&(!this.queryResult.isFulfilled())){
                this.queryResult.cancel();
            }
            var queryResult = this.store.query(this.queryParam,{
                start:this.start,
                count:this.pageSize
            });
            queryResult.then(lang.hitch(this,this.onDataReady));
            queryResult.total.then(lang.hitch(this,this.onTotalReady));
            return queryResult;
        },
        getCqQueryInfo:function(){
            return {
                target:this.store&&this.store.target,
                query:this.queryParam
            };
        },
        canReload:function(){
            // summary:
            //      判断是否可以刷新，解决短时间内多重刷新问题
            return this._lastQueryStart===this.start&&JSON.stringify(this.queryParam)===this._lastQueryParam&&2000>(new Date())-this._lastQueryTime?false:true;
        },
        //初始化列表
        initList:function(){
            // summary:
            //      初始化列表，内容的清理，从第几页加载，查询条件等
            this.start = 0;
            this.selectedItems = [];
            if(this.store){
                var query = {};
                if(this.dataProvider[this.queryParamMethod]){
                    query = this.dataProvider[this.queryParamMethod](this.cqRefInfo);
                }
                this.queryParam = query;
                this.store.target = this.dataProvider[this.targetMethod]();
            }
            this.clearList();
        },
        clearList:function(){
            if(this.fieldItem&&this.fieldItem.domNode.parentNode){
                this.fieldItem.domNode.parentNode.removeChild(this.fieldItem.domNode);
            }
            var children = this.getChildren();
            while(children.length){
                children.pop().destroyRecursive();
            }
            if(this.fieldItem){
                this.addChild(this.fieldItem);
            }
            if(this.allData){
                this.addChild(new this._itemRender(this._getItemParam(this.allData)));
            }
        },
        _getItemParam:function(data){
            if(this.dataProvider&&this.itemPropMethod){
                data = this.dataProvider[this.itemPropMethod](data,this.itemType);
            }
            if(!data.data){
                data.data = data;
            }
            data.layout = this.layout;
            return data;
        },
        //数据接在完成
        onDataReady:function(datas){
            if(datas.items){
                datas = datas.items;
            }
            array.forEach(datas,function(data){
                data = this._getItemParam(data);
                this.addChild(new this._itemRender(data));
            },this);
            //处理默认选择
            if(this.selected){
                for(var i=0; i<this.domNode.childNodes.length;i++){
                    if(this.domNode.childNodes[i].innerHTML===this.selected){
                        //registry.byId(this.domNode.childNodes[i].id).set("checked",true);    
                        this.onItemClick(registry.byId(this.domNode.childNodes[i].id));
                        break;
                    }
                }
            }
            this.onCqChanged(this,null);
            var view = this.getParent();
            while(view&&(!view.show)){
                view = view.getParent();
            }
            if(true===this.hideWhenNoData){
                var hide = datas.length===0&&0===this._lastQueryStart;
                domStyle.set(this.domNode,"display",hide?"none":"");
                if(this.labelNode&&this.label){
                    domStyle.set(this.labelNode,"display",hide?"none":"");
                }
                if(this.pagerNode){
                    domStyle.set(this.pagerNode,"display",hide?"none":"");
                }
            }
            if(view){
                view.resize();
            }
        },
        //处理分页，数据总行数
        onTotalReady:function(total){
            
        },
        //初始化选中的列表项
        _initSelected:function(){
            this.selectedItems = this.selectedItems||[];
            while(this.selectedItems.length){
                this.selectedItems.pop().set("checked",false);
            }
        },
        //获取节点对应的操作方法
        getActionByNode:function(item,node){
            // summary:
            //      依据节点获取操作类型，clickMethod中可以依据不同的操作类型做不同的相应
            
            var action = "";
            while(node&&node.parentNode&&node!==item.domNode){
                action = domAttr.get(node,"cq-action");
                if(action){
                    break;
                }
                node = node.parentNode;
            }
			//干劲：少return
			return action;
        },
        //列表项点击之后的响应
        onItemClick:function(item,e){
            // summary:
            //      点击的处理函数，处理了列表在不同工作模式下的不同响应方式
            //不是多选，清空选中
            if("multi"!=this.workMode){
                this._initSelected();
            }
            //设置新的选中
            this.selectedItems.push(item);
            item.set("checked",true);
            //依据工作模式做出不同的响应
            switch(this.workMode){
                case "single":
                    //单选
                    this.onCqChanged(this,item.params.data);
                    break;
                case "multi":
                    //多选
                    break;
                default:
                    //点击响应
                    this.dataProvider[this.clickMethod](item.params.data,e?this.getActionByNode(item,e.target):"",this);
                    break;
            }
        },
        removeItemByData:function(data){
            // summary:
            //      依据数据销毁对应的列表项
            if(data){
                array.some(this.getChildren(),function(item){
                    if(item.params.data&&item.params.data._id === data._id){
                        item.destroyRecursive();
                        return true;
                    }
                },this);
            }
        },
        refreshItemByData:function(data){
            // summary:
            //      依据数据刷新对应的列表项
            if(data){
                array.some(this.getChildren(),function(item){
                    if(item.params.data&&item.params.data._id === data._id){
                        data = this._getItemParam(data);
                        (new this._itemRender(data)).placeAt(item.domNode,"before");
                        item.destroyRecursive();
                        return true;
                    }
                },this);
            }
        },
        cqRefresh:function(){
            //      当监听到其他对象变化时，刷新本对象
            this.refresh();
            
            /*if(this.class==="cq-list "&&this.domNode.childNodes.length){
                this.onItemClick(registry.byId(this.domNode.childNodes[0].id));
            }*/
        },
        getSelectedItems:function(){
            // summary:
            //      获取当前选中的列表项的数据
            if("single"===this.workMode){
                return this.selectedItems&&this.selectedItems.length?this.selectedItems[0].params.data:null;
            }else{
                return array.map(this.selectedItems,function(item){
                    return item.params.data;
                });
            }
        }
    });
});