﻿//本对象为所有对象的基类
//  处理对象联动的机制和接口
define([
    "dojo/_base/declare",
    "dojo/_base/lang"
],function(declare,lang){
    // module:
    //      cq/app/widget/_WidgetBase
    
    return declare("",null,{
        // summary:
        //      所有widget的基类，为对象间提供联动刷新机制
        
        // cqRefInfo:object
        //      监听到的属性和属性值
        // tags:
        //      readOnly
        cqRefInfo:null,
        
        // cqWaitInfo:object
        //      监听的对象的id和属性之间的对照关系
        // tags:
        //      private
        cqWaitInfo:null,
        
        // _cqObsHandler:Array
        //      监听其他对象的事件对象数组
        // tags:
        //      private
        _cqObsHandler:null,
        
        buildRendering:function(){
            // summary:
            //      创建对象时，初始化对象联动刷新相关的属性
            
            if(this._cqObsHandler){
                while(this._cqObsHandler.length){
                    this._cqObsHandler.pop().remove();
                }
            }
            this._cqObsHandler = [];
            this.cqRefInfo = {};
            this.cqWaitInfo = {};
            
            // 执行默认操作
            this.inherited(arguments);
        },
        observe:function(widget,field){
            // summary:
            //      监听一个对象的变化
            //      刷新自己的cqRefInfo，同时触发自己的刷新方法
            if(this.cqRefInfo[widget.get("id")]){
                return;
            }
            this.cqWaitInfo[widget.get("id")] = field;
            this._cqObsHandler.push(this.connect(widget,"onCqChanged",this._onObserveChanged));
        },
        _onObserveChanged:function(widget,value){
            // summary:
            //      自己监听的对象发生变化之后的相应事件
            // tags:
            //      private
            this.cqRefInfo[this.cqWaitInfo[widget.get("id")]] = value;
            
            // 在短时间内，多个其他对象触发本对象刷新，只相应一次
            clearTimeout(this._cqRefreshTimer);
            this._cqRefreshTimer = setTimeout(lang.hitch(this,this.cqRefresh),500);
        },
        cqRefresh:function(){
            // summary:
            //      其他对象触发的本对象的刷新动作
            //      最好刷新的接口只有一个
            //      若要相应其他对象的变化，就必须实现这个方法
            // tags:
            //      interface
        },
        //自己发生变化之后发出的事件，供其他对象侦听和响应
        onCqChanged:function(widget,value){
            // summary:
            //      本对象发生变化之后通知其他对象刷新的接口
            //      本接口只用于调用，不用在子类中实现
            // params:
            //      widget:当前对象
            //      value:当前对象当前的值(需要通知其他对象的内容)
        }
    });
});