define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/Deferred",
    "dijit/_WidgetBase",
    "dijit/form/DateTextBox"
],function(declare,lang,domConstruct,Deferred,_WidgetBase,DateTextBox){
    // module:
    //      cq/app/widget/web/edit/DatePicker  
    
    return declare("cq.app.widget.web.edit.DatePicker",[_WidgetBase],{
        // summary: 
        //      定义了web端的日期组件(不含小时)，继承自_WidgetBase.
        
        // htmlTemp:String
        //      组件的DOM结构字符串,并传递组件的属性。
        // tags:
        //      private
        htmlTemp:'<div id="{id}" data-cq-field="{field}" class="cq-field-input cq-form-DatePicker cq-form-input" data-dojo-type="cq/app/widget/web/edit/DatePicker" data-dojo-props="canEdit:{canEdit},field:\'{field}\'"></div>',
        
        // field:String
        //      组件名称
        //      实际可能是该组件需要的某些字段、组件的组件名、与之联动的组件的组件名或字段名构成的一段字符串。
        field:"",
        
        // canEdit:boolean
        //      编辑状态，true为可编辑。
        canEdit:true,
        
        buildRendering:function(){
            // summary:
            //      初始日期组件。
            //tags:
            //      private
            
            this.inherited(arguments);
            this.containerNode = this.containerNode||this.domNode;
            this._initPicker();
        },
        setCanEdit:function(canEdit){
            // summary:
            //      设置组件是否可编辑。
            // params:
            //      canEdit:编辑状态
            // tags:
            //      public
            
            this.canEdit = canEdit;
            if(this.canEdit == false){
                this.box.set("disabled", "disabled");
            }else{
                this.box.set("disabled", "");
            }
        },
        
        _initPicker:function(){
            // summary:
            //      初始化一个月份日期窗口,加入到组件节点中。
            // tags:
            //      private
            if(!this.box){
                this.box = new DateTextBox({
                    value:this.getValue(),
                    onChange:lang.hitch(this, function(value){
                        this.setValue(value);
                        this.onFieldChanged(this, this.field, value);
                    })
                });
                this.box.placeAt(this.containerNode, "last");
                //设置是否可编辑。
                if(this.canEdit == false){
                    this.box.set("disabled", "disabled");
                }
            }else{
                this.setValue(this.getValue());
            }
        },
        
        onFieldChanged:function(widget, field, value){
            // summary:
            //      本对象发生变化后通知其他对象刷新的接口
            // params:
            //      widget:发生变化对象所在的view
            //      field:发生变化的对象
            //      value:改变后的值
            // tags:
            //      interface
        },
        setValue:function(date){
            // summary:
            //      设置组件的值
            // params:
            //      date:组件的值（一个日期）
            // tags:
            //      private
            this.value = date||(new Date());
            if(this.box){
                this.box.setValue(this.value);
            }
        },
        getValue:function(){
            // summary：
            //      获取组件的值
            // tags:
            //      private
            return this.value||new Date();
        }
    });
});