define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/Deferred",
    "dijit/_WidgetBase",
    "dijit/form/DateTextBox",
    "dijit/form/TimeTextBox",
    "cq/util/sys/date"
],function(declare,lang,domConstruct,Deferred,_WidgetBase,DateTextBox,TimeTextBox,date){
    // module:
    //      cq/app/widget/web/edit/DateTimePicker 
    
    return declare("cq.app.widget.web.edit.DateTimePicker",[_WidgetBase],{
        // summary: 
        //      定义了web端的日期组件(包含小时)，继承自_WidgetBase.
        
        // htmlTemp:String
        //      组件的DOM结构字符串,并传递组件的属性。
        // tags:
        //      private
        htmlTemp:'<div id="{id}" data-cq-field="{field}" class="cq-field-input cq-form-DateTimePicker cq-form-input" data-dojo-type="cq/app/widget/web/edit/DateTimePicker" data-dojo-props="canEdit:{canEdit},field:\'{field}\'"></div>',
        
        // field:String
        //      组件名称
        //      实际可能是该组件需要的某些字段、组件的组件名、与之联动的组件的组件名或字段名构成的一段字符串。
        field:"",
        
        // canEdit:boolean
        //      编辑状态，true为可编辑。
        canEdit:true,
        buildRendering:function(){
            // summary:
            //      初始日期组件。
            //tags:
            //      private
            
            this.inherited(arguments);
            this.containerNode = this.containerNode||this.domNode;
            this._initPicker();
        },
        setCanEdit:function(canEdit){
            // summary:
            //      设置组件是否可编辑。
            // params:
            //      canEdit:编辑状态
            // tags:
            //      public
            
            this.canEdit = canEdit!==false&&canEdit!=="false";
            /*
            if(this.canEdit == false){
                this.box.set("disabled", "disabled");
            }else{
                this.box.set("disabled", "");
            }*/
            this.box.set("disabled",this.canEdit?"":"disabled");
            this.timeBox.set("disabled",this.canEdit?"":"disabled");
        },
        _initPicker:function(){
            // summary:
            //      分别初始化一个日期和时间窗口,加入到组件节点中。
            // tags:
            //      private
            if(!this.box){
                this.box = new DateTextBox({
                    value:this.getValue(),
                    onChange:lang.hitch(this, function(value){
                        this.setDateValue(value);
                        this.onFieldChanged(this, this.field, this.getValue());
                    })
                });
                this.box.placeAt(this.containerNode, "last");
                if(this.canEdit == false){
                    this.box.set("disabled", "disabled");
                }
                this.timeBox = new TimeTextBox({
                    value:this.getValue(),
                    onChange:lang.hitch(this, function(value){
                        this.setTimeValue(value);
                        this.onFieldChanged(this, this.field, this.getValue());
                    })
                });
                this.timeBox.placeAt(this.containerNode, "last");
                if(this.canEdit == false){
                    this.timeBox.set("disabled", "disabled");
                }
            }else{
                this.setValue(this.getValue());
            }
        },
        
        onFieldChanged:function(widget, field, value){
            // summary:
            //      本对象发生变化后通知其他对象刷新的接口
            // params:
            //      widget:发生变化对象所在的view
            //      field:发生变化的对象
            //      value:改变后的值
            // tags:
            //      interface
        },
        setDateValue:function(value){
            // summary:
            //      设置日期窗口的值
            // params:
            //      value:日期的值
            // tags:
            //      private
            
			if(!value) value = new Date();
			this.dateValue = date.formatISODate(value);
			if(!this.value) {
				this.value = value.toISOString();
				this.timeValue = date.formatISOTime(value);
			}
			else this.value = date.formatToISOStringWithMinutes(this.dateValue+" "+this.timeValue);
            if(this.box){
                this.box.setValue(value);
            }
        },
        setTimeValue:function(value){
            // summary:
            //      设置时间窗口的值
            // params:
            //      时间的值
            // tags:
            //      private
            
			if(!value) value = new Date();
			this.timeValue = date.formatISOTime(value);
			if(!this.value) {
				this.value = value.toISOString();
				this.dateValue = date.formatISODate(value);
			}
			else this.value = date.formatToISOStringWithMinutes(this.dateValue+" "+this.timeValue);
            if(this.timeBox){
                this.timeBox.setValue(value);
            }
        },
        setValue:function(value){
            // summary:
            //      设置组件的值
            // params:
            //      组件的值
            // tags:
            //      private
            
            value = value||(new Date()).toISOString();
			this.value = value;
			var d = new Date(value);
			this.timeValue = date.formatISOTime(value);
			this.dateValue = date.formatISODate(value);
            if(this.box){
                this.box.setValue(d);
            }
            if(this.timeBox){
                this.timeBox.setValue(d);
            }
        },
        getValue:function(){
            // summary:
            //      获取组件的值
            // tags:
            //      private
            return this.value||(new Date()).toISOString();
        }
    });
});