define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/dom-class",
    "dojo/dom-attr",
    "dojo/Deferred",
    "dijit/_WidgetBase",
    "cq/biz/shop/bs/district"
],function(declare,lang,array,domConstruct,domStyle,domClass,domAttr,Deferred,_WidgetBase,district){
    // module:
    //      cq/app/widget/web/edit/DistrictEdit
    
    return declare("cq.app.widget.web.edit.DistrictEdit",[_WidgetBase],{
        // summary: 
        //      定义了web端的区划组件，继承自_WidgetBase.
        
        // htmlTemp:String
        //      组件的DOM结构字符串,并传递组件的属性。
        // tags:
        //      private
        htmlTemp:'<div id="{id}" data-cq-field="{field}" class="cq-field-input cq-form-districtedit" data-dojo-type="cq/app/widget/web/edit/DistrictEdit" data-dojo-props=\'field:"{field}",fixedUpDis:{j:fixedUpDis},canEdit:"{canEdit}"\'></div>',
        
        // field:String
        //      组件名称
        //      实际可能是该组件需要的某些字段、组件的组件名、与之联动的组件的组件名或字段名构成的一段字符串。
        field:'',
        
        // canEdit:boolean
        //      编辑状态，true为可编辑。
        canEdit:true,
        
        // fixedUpDis:string
        //      如果只需要选择区县或者市区，而上级区划固定，则需要使用该属性，该属性的值为上级固定区划的级别(province/city)
        fixedUpDis:null,
        
        // districtHash:Object
        //      区划对象表
        // tags:
        //      private
        districtHash:{},
        
        buildRendering:function(){
            // summary:
            //      初始区划组件。
            //tags:
            //      private
            this.inherited(arguments);
            this.containerNode = this.containerNode||this.domNode;
            this._initEdit();
        },
        
        setCanEdit:function(canEdit){
            // summary:
            //      设置组件是否可编辑。
            // params:
            //      canEdit:编辑状态
            // tags:
            //      public
            this.canEdit = canEdit;
            if(this.canEdit === false){
                domAttr.set(this.proSelect, "disabled", "disabled");
                domAttr.set(this.citySelect, "disabled", "disabled");
                domAttr.set(this.countrySelect, "disabled", "disabled");
            }else{
                domAttr.remove(this.proSelect, "disabled");
                domAttr.remove(this.citySelect, "disabled");
                domAttr.remove(this.countrySelect, "disabled");
            }
        },
        _initEdit:function(){
            // summary:
            //      初始化组件中的下拉列表
            //      加载省市区数据，并设置到下拉列表
            //      当区划发生改变时通知与之联动的组件
            // tags:
            //      private
            if(!this.editor){
                this.editor = domConstruct.create("div", {
                    className:"cq-districteditor"
                }, this.containerNode);
                this.proSelect = domConstruct.create("select", {
                    className:"cq-districteditor-select"
                }, this.editor);
                this.citySelect = domConstruct.create("select", {
                    className:"cq-districteditor-select"
                }, this.editor);
                this.countrySelect = domConstruct.create("select", {
                    className:"cq-districteditor-select"
                }, this.editor);
                
                // 省份数据，启动时加载
                this.loadDistrictData(2).then(lang.hitch(this, function(data){
                    domConstruct.create("option", {
                        innerHTML:"请选择省份"
                    }, this.proSelect);
                    data.forEach(lang.hitch(this, function(dis){
                        domConstruct.create("option", {
                            innerHTML:dis.distName,
                            _id:dis._id
                        }, this.proSelect);
                    }));
                    
                    //  张松   如果只选择市区或者区县，隐藏上级区划select     2016/11/7
                    if(this.fixedUpDis.upDis&&this.fixedUpDis.upDis.length){
                        this.setValue(this.fixedUpDis.upDis);
                        domStyle.set(this.proSelect, "display", "none");
                        if(this.fixedUpDis.upDis.length == 2){
                            domStyle.set(this.citySelect, "display", "none");
                        }
                    }
                }));
                domConstruct.create("option", {
                    innerHTML:"全部"
                }, this.citySelect);
                domConstruct.create("option", {
                    innerHTML:"全部"
                }, this.countrySelect);
                
                // 添加监听，当选中状态发生变化时，下级内容发生变化，onFieldChanged， value变化
                this.connect(this.proSelect, "onchange", lang.hitch(this, function(evt){
                    // 删除默认的选项，省份必须选择
                    if(this.proSelect.options[0].innerHTML == "请选择省份"){
                        this.proSelect.removeChild(this.proSelect.options[0]);
                    }
                    this.citySelect.innerHTML = "";
                    domConstruct.create("option", {
                        innerHTML:"全部"
                    }, this.citySelect);
                    this.countrySelect.innerHTML = "";
                    domConstruct.create("option", {
                        innerHTML:"全部"
                    }, this.countrySelect);
                    // 加载市的内容
                    this.loadDistrictData(domAttr.get(this.proSelect.selectedOptions[0] ,"_id")).then(lang.hitch(this, function(data){
                        data.forEach(lang.hitch(this, function(dis){
                            domConstruct.create("option", {
                                innerHTML:dis.distName,
                                _id:dis._id
                            }, this.citySelect);
                        }));
                    }));
                    // value值
                    this.value[0] = this.districtHash[2][this.proSelect.selectedIndex];
                    this.value[1] = null;
                    this.value[2] = null;
                    this.value[3] = this.value[0]?(this.value[0].distName + (this.value[1]?("," + this.value[1].distName + (this.value[2]?("," + this.value[2].distName):"")):"")):"";
                    // 触发onFieldChanged
                    this.onFieldChanged(this, this.field.split(",")[0], this.value[0]);
                    this.onFieldChanged(this, this.field.split(",")[1], this.value[1]);
                    this.onFieldChanged(this, this.field.split(",")[2], this.value[2]);
                    this.onFieldChanged(this, this.field.split(",")[3], this.value[3]);
                }));
                this.connect(this.citySelect, "onchange", lang.hitch(this, function(evt){
                    this.countrySelect.innerHTML = "";
                    domConstruct.create("option", {
                        innerHTML:"全部",
                    }, this.countrySelect);
                    // 如果选择全部,则县也选择全部,清除县中所有数据
                    if(this.proSelect.options[0].innerHTML == "全部"){
                        this.value[1] = null;
                    }else{
                        this.loadDistrictData(domAttr.get(this.citySelect.selectedOptions[0], "_id")).then(lang.hitch(this, function(data){
                            data.forEach(lang.hitch(this, function(dis){
                                domConstruct.create("option", {
                                    innerHTML:dis.distName,
                                    _id:dis._id
                                }, this.countrySelect);
                            }));
                        }));
                        this.value[1] = this.districtHash[domAttr.get(this.proSelect.selectedOptions[0], "_id")][this.citySelect.selectedIndex - 1];
                    }
                    this.value[2] = null;
                    this.value[3] = this.value[0]?(this.value[0].distName + (this.value[1]?("," + this.value[1].distName + (this.value[2]?("," + this.value[2].distName):"")):"")):"";
                    this.onFieldChanged(this, this.field.split(",")[1], this.value[1]);
                    this.onFieldChanged(this, this.field.split(",")[2], this.value[2]);
                    this.onFieldChanged(this, this.field.split(",")[3], this.value[3]);
                }));
                this.connect(this.countrySelect, "onchange", lang.hitch(this, function(evt){
                    this.value[2] = this.districtHash[domAttr.get(this.citySelect.selectedOptions[0], "_id")][this.countrySelect.selectedIndex - 1];
                    this.value[3] = this.value[0]?(this.value[0].distName + (this.value[1]?("," + this.value[1].distName + (this.value[2]?("," + this.value[2].distName):"")):"")):"";
                    this.onFieldChanged(this, this.field.split(",")[2], this.value[2]);
                    this.onFieldChanged(this, this.field.split(",")[3], this.value[3]);
                }));
            }else{
                this.editor.setValue(this.getValue());
            }
        },
        loadDistrictData:function(parentId){
            // summary: 
            //      加载区划数据
            // params:
            //      parentId:上级id,省份时ParentId传为2
            // tags:
            //      private
            parentId = parentId||2;
            if(this.districtHash[parentId]){
                var def = new Deferred();
                def.resolve(this.districtHash[parentId]);
                return def;
            }else{
                return district[2===parentId?"loadByLevel":"loadByUpId"](parentId).then(lang.hitch(this, function(data){
                    data = data&&data.data&&data.data.items||[];
                    this.districtHash[parentId] = data;
                    return data;
                }));
            }
        },
        onFieldChanged:function(widget, field, value){
            // summary:
            //      本对象发生变化后通知其他对象刷新的接口
            // params:
            //      widget:发生变化对象所在的view
            //      field:发生变化的对象
            //      value:改变后的值
            // tags:
            //      interface
        },
        setValue:function(value){
            // summary:
            //      设置区划组件的值
            // params:
            //      value:区划组件的值
            // tags:
            //      private
            this.value = value||[];
            if(this.fixedUpDis&&this.fixedUpDis.upDis){
                for(var i=0;i<this.fixedUpDis.upDis.length;i++){
                    if(this.value[i]){
                        break;
                    }else{
                        this.value[i] = this.fixedUpDis.upDis[i];
                    }
                }
            }
            // 设置省的选中项
            if(value[0]){
                // 设置省市的默认选中项，需要先在buildRendering中先加载省数据，判断省是否加载完成
                var options = this.proSelect.options;
                if(!options || !options.length){
                    return;
                }
                array.forEach(options, lang.hitch(this, function(option, index){
                    if(domAttr.get(option, "_id") == (value[0]._id||value[0])){
                        this.proSelect.selectedIndex = index;
                    }
                }));
                this.citySelect.innerHTML = "";
                domConstruct.create("option", {
                    innerHTML:"全部"
                }, this.citySelect);
                this.countrySelect.innerHTML = "";
                domConstruct.create("option", {
                    innerHTML:"全部"
                }, this.countrySelect);
                this.loadDistrictData(domAttr.get(this.proSelect.selectedOptions[0] ,"_id")).then(lang.hitch(this, function(data){
                    data.forEach(lang.hitch(this, function(dis){
                        domConstruct.create("option", {
                            innerHTML:dis.distName,
                            _id:dis._id
                        }, this.citySelect);
                    }));
                    if(value[1]){
                        // 设置市的选中项
                        options = this.citySelect.options;
                        array.forEach(options, lang.hitch(this, function(option, index){
                            if(domAttr.get(option, "_id") == (value[1]._id||value[1])){
                                this.citySelect.selectedIndex = index;
                            }
                        }));
                        this.loadDistrictData(domAttr.get(this.citySelect.selectedOptions[0], "_id")).then(lang.hitch(this, function(data){
                            data.forEach(lang.hitch(this, function(dis){
                                domConstruct.create("option", {
                                    innerHTML:dis.distName,
                                    _id:dis._id
                                }, this.countrySelect);
                            }));
                            // 设置县的选中项
                            if(value[2]){
                                options = this.countrySelect.options;
                                array.forEach(options, lang.hitch(this, function(option, index){
                                    if(domAttr.get(option, "_id") == (value[2]._id||value[2])){
                                        this.countrySelect.selectedIndex = index;
                                    }
                                }));
                            }
                        }));
                    }
                }));
            }
        },
        getValue:function(){
            // summary:
            //      获取区划组件的值
            // tags:
            //      private
            return this.value||[];
        }
    });
});