define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dojo/dom-attr",
    "dojo/dom-style",
    "dojo/query",
    "dojo/Deferred",
    "dojo/touch",
	"dojo/dnd/Source",
    "dijit/_WidgetBase",
    "cq/widgets/bigImg",
    "cq/biz/blob/blob"
],function(declare,lang,array,domConstruct,domClass,domAttr,domStyle,query,Deferred,touch,Source,_WidgetBase,bigImg,blob){
    //module:
    //      cq/app/widget/web/edit/ImagePicker
    return declare("cq.app.widget.web.edit.ImagePicker",[_WidgetBase],{
        // summary: 
        //      定义了web端的添加图片组件，继承自_WidgetBase.
        
        // htmlTemp:String
        //      组件的DOM结构字符串,并传递组件的属性。
        // tags:
        //      private
        htmlTemp:'<div id="{id}" data-cq-field="{field}" class="cq-field-input cq-form-ImagePicker" data-dojo-type="cq/app/widget/web/edit/ImagePicker" data-dojo-props=\'field:"{field}",mode:"{mode}",canEdit:"{canEdit}",sizeType:"{sizeType}",size:{j:size}\'></div>',
        
        // field:String
        //      组件名称
        //      实际可能是该组件需要的某些字段、组件的组件名、与之联动的组件的组件名或字段名构成的一段字符串。
        field:"",
        // mode:String
        //      图片组件的模式 --- 单图(singular)、多图(duplicate)
        mode:"singular",
        
        // size:Object
        //      图片的大小,例如：size{width:128,height128}
        size:{},
        
        // sizeType:String
        //      图片压缩方式，剪切("cut")
        //      默认为压缩
        sizeType:"",
        
        // canEdit:boolean
        //      编辑状态，true为可编辑。
        canEdit:true,
        buildRendering:function(){
            // summary:
            //      初始化组件
            // tags:
            //      private
            this.canEdit = this.canEdit!==false&&this.canEdit!=="false";
            this.inherited(arguments);
            /*
            if(this.size.width||this.size.height){
            }else{
                this.size = {width:128,height:128};
            }*/
            this.containerNode = this.containerNode||this.domNode;
            this._initPicker();
        },
        _initPicker:function(){
            // summary:
            //      定义组件的添加图片按钮，删除图片按钮
            //      实现图片压缩剪切
            //      绑定按钮和图片的点击事件
            // tags:
            //      private
            if(!this.picker){
                this.picker = domConstruct.create("div",{
                    className:"cq-imgpicker"
                }, this.containerNode);
				this.dndSource = new Source(this.picker,{horizontal:true});
                //this.imgPane = domConstruct.create("div", {
                //    className:"cq-imgpicker-imgpane"
                //}, this.picker);
                this.addPane = domConstruct.create("div", {
                    innerHTML:"+",
                    className:"cq-imgpicker-addpane"
                }, this.picker);
                //this.img = domConstruct.create("img", {
                //    className:"cq-imgpicker-img",
                //    src:this.getValue()
                //}, this.imgPane);
                if(this.canEdit == false){
                    domStyle.set(this.addPane, "display", "none");
                }
                //touch.release(this.picker, lang.hitch(this, function(e){
                //不可编辑时图片节点的点击事件
				this.picker.onclick=lang.hitch(this, function(e){
                    if(this.canEdit == false){
						if(e.target.tagName.toLowerCase() == "img"){
							bigImg.show(e.target.src);
						}
                        return;
                    }
                    var target = e.target;
                    //可编辑时图片节点的点击事件
                    if(target.tagName.toLowerCase() == "img"){
						//return;
                        // 如果点击的是图片节点，则替换原来的图片
                        blob.selectAndUpload(this.size,0.7,this.sizeType).then(lang.hitch(this, function(data){
                            var datas = "";
                            if(this.mode == "duplicate"){
                                target.src = data.items[0].uri;
                                var imgs = query("img", this.picker);
                                datas = [];
                                array.forEach(imgs, function(item, index){
                                    datas[index] = item.src;
                                });
                                this.editing = true;
                                this.setValue(datas);
                            }else{
                                this.setValue(data.items[0].uri);
                                datas = data.items[0].uri;
                            }
                            this.onFieldChanged(this, this.field, datas);
                        }));
                    }else if(domClass.contains(target, "cq-imgpicker-addpane")){
                        // 如果点击的是+，则新增一张图片
                        blob.selectAndUpload(this.size,0.7,this.sizeType).then(lang.hitch(this, function(data){
                            var datas = "";
                            if(this.mode == "duplicate"){
                                datas = [];
                                var con = domConstruct.create("div", {
                                    className:"cq-imgpicker-imgcontainer dojoDndItem"
                                }, this.picker, query("img", this.picker).length);
                                domConstruct.create("img", {
                                    src:data.items[0].uri,
                                    className:"cq-imgpicker-img"
                                }, con);
                                var del = domConstruct.create("div", {
                                    className:"cq-imgpicker-imgdelete"
                                }, con);
                                domConstruct.create("div", {
                                    innerHTML:"+"
                                }, del);
                                var imgs = query("img", this.picker);
                                array.forEach(imgs, function(item, index){
                                    datas[index] = item.src;
                                });
								this.dndSource.insertNodes(false,[con],true,this.addPane);
                                this.editing = true;
                                this.setValue(datas);
                            }else{
                                this.setValue(data.items[0].uri);
                                datas = data.items[0].uri;
                            }
                            this.onFieldChanged(this, this.field, datas);
                        }));
                    }else if(domClass.contains(target, "cq-imgpicker-imgdelete") || domClass.contains(target.parentNode, "cq-imgpicker-imgdelete")){
                        // 如果点击的是x，则删除该图片
                        if(domClass.contains(target, "cq-imgpicker-imgdelete")){
                            target = target.parentNode;
                        }else if(domClass.contains(target.parentNode, "cq-imgpicker-imgdelete")){
                            target = target.parentNode.parentNode;
                        }
                        target.parentNode.removeChild(target);
                        var imgs = query("img", this.picker);
                        var datas = [];
                        array.forEach(imgs, function(item, index){
                            datas[index] = item.src;
                        });
                        this.setValue(datas);
                        this.onFieldChanged(this, this.field, datas);
                    }
                });
            }else{
                this.setValue(this.getValue());
            }
        },
        onFieldChanged:function(widget, field, value){
            // summary:
            //      本对象发生变化后通知其他对象刷新的接口
            // params:
            //      widget:发生变化对象所在的view
            //      field:发生变化的对象
            //      value:改变后的值
            // tags:
            //      interface
        },
        setCanEdit:function(canEdit){
            // summary:
            //      设置是否可编辑
            // params:
            //      canEdit:编辑状态   
            this.canEdit = canEdit;
            if(this.canEdit == false){
                domStyle.set(this.addPane, "display", "none");
            }else{
                domStyle.set(this.addPane, "display", "inline-block");
            }
        },
        onImageChanged:function(data){
            // summary:
            //      添加或替换图片
            // params:
            //      data:用于替换的图片URL
            
            //判断是单图或多图模式
            if(this.mode == "duplicate"){
                // 删除原有图片，新增图片
                // 如果是第一次进入页面，设置内容，editing为false，新增图片
                // 如果不是第一次进入，editing为true，不做修改
                if(this.editing){
                    this.editing = false;
                    return;
                }
                var imgs = query(".cq-imgpicker-imgcontainer", this.picker);
                array.forEach(imgs, lang.hitch(this, function(item, index){
                    this.picker.removeChild(item);
                }));
                array.forEach(data, lang.hitch(this, function(item, index){
                    var con = domConstruct.create("div", {
                        className:"cq-imgpicker-imgcontainer"+(this.canEdit?" dojoDndItem":"")
                    }, this.picker, index);
                    domConstruct.create("img", {
                        src:item,
                        className:"cq-imgpicker-img"
                    }, con);
                    var del = domConstruct.create("div", {
                        className:"cq-imgpicker-imgdelete"
                    }, con);
                    domConstruct.create("div", {
                        innerHTML:"+"
                    }, del);
					this.dndSource.insertNodes(false,[con],true,this.addPane);
                }));
            }else{
                // 如果有图片，替换
                // 如果没有图片，创建图片,并且隐藏+
                if(this.img){
                    if(data){
                        this.img.src = data;
                    }else{
                        this.img.parentNode.removeChild(this.img);
                        this.img = null;
                        domStyle.set(this.addPane, "display", "inline-block");
                    }
                }else{
                    if(data){
                        this.img = domConstruct.create("img", {
                            src:data,
                            className:"cq-imgpicker-img"
                        }, this.picker);
                        domStyle.set(this.addPane, "display", "none");
                    }else{
                        domStyle.set(this.addPane, "display", "inline-block");
                    }
                }
            }
        },
        setValue:function(value){
            // summary:
            //      设置组件的值
            // params:
            //      value:组件的值
            // tags:
            //      private
            if(this.mode == "duplicate"){
                this.value = value||[];
            }else{
                this.value = value||"";
            }
            if(this.picker){
                this.onImageChanged(this.value);
            }
        },
        getValue:function(){
            // summary:
            //      获取组件的值
            // tags:
            //      private
            if(this.mode == "duplicate"){
				var value = [];
				var imgs = query("img",this.picker);
				array.forEach(imgs, function(item, index){
                                    value[index] = item.src;
                                });
                return value;
            }else{
                return this.value||"";
            }
        }
    });
});