define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/query",
    "dojo/dom-construct",
    "dojo/dom-attr",
    "cq/app/widget/web/DataLayout",
    "cq/bo/BoBase",
    "dijit/_WidgetBase"
], function(declare, lang, query, domConstruct, domAttr, DataLayout, BoBase, _WidgetBase){
    // module: 
    //      ca/app/widget/web/edit/ObjEditor
    return declare("cq.app.widget.web.edit.ObjEditor",[_WidgetBase], {
        // summary: 
        //      定义了web端的容器组件，该组件中可以放入其他组件，继承自_WidgetBase.
        
        // htmlTemp:String
        //      组件的DOM结构字符串,并传递组件的属性。
        // tags:
        //      private
        htmlTemp:'<div id="{id}" data-cq-field="{field}" class="cq-field-input cq-form-ObjEditor" data-dojo-type="cq/app/widget/web/edit/ObjEditor" data-dojo-props=\'fields:{j:fields},canEdit:"{canEdit}",field:"{field}"\'></div>',
        
        // field:String
        //      组件名称
        //      实际可能是该组件需要的某些字段、组件的组件名、与之联动的组件的组件名或字段名构成的一段字符串。
        field:"",
        
        // fields:Object
        //      对象要实现的组件集合
        fields:{},
        
        // canEdit:boolean
        //      编辑状态，true为可编辑。
        canEdit:true,
        
        buildRendering:function(){
            // summary:
            //      初始化组件
            // tags:
            //      private
            this.canEdit = this.canEdit!==false&&this.canEdit!=="false";
            this.fields = this.fields.layout;
            this.inherited(arguments);
            this.containerNode = this.containerNode||this.domNode;
            setTimeout(lang.hitch(this, function(){
                this._initEditor();
            }),0);
        },
		
        _initEditor:function(){
            // summary:
            //      根据this.fields实例化要在容器中实现的其他组件
            // tags:
            //      private
			if(this.editor){
				this.editor.destroyRecursive();
			}
			var layout = {};
			var fields = [];
			lang.clone(this.fields).forEach(function(item){
				fields.push(item.field);
				layout[item.field] = item;
				delete layout[item.field]["field"];
			});
			var model = new BoBase({
				fields:layout
			});
			this.editor = new DataLayout({
				layout:[{
					type:"form",
					fields:model.getFormItems(fields)
				}]
			});
			this.editor.placeAt(this.containerNode);
			this.editor.connect(this.editor, "onWidgetFieldChanged", lang.hitch(this, function(widget, fields, values){
				this.value[fields] = values;
				this.onFieldChanged(this, this.field, this.value);
			}));
			this.setCanEdit(this.canEdit);
			this.setValue(this.getValue());
        },
        onFieldChanged:function(widget, field, value){
            // summary:
            //      本对象发生变化后通知其他对象刷新的接口
            // params:
            //      widget:发生变化对象所在的view
            //      field:发生变化的对象
            //      value:改变后的值
            // tags:
            //      interface
        },
        setCanEdit:function(canEdit){
            // summary:
            //      设置组件是否可编辑。
            // params:
            //      canEdit:编辑状态
            // tags:
            //      public
            this.canEdit = canEdit!==false&&canEdit!=="false";
            if(this.canEdit == false){
                if(this.editor){
                    this.editor.fieldElements.forEach(function(node){
                        node.setCanEdit(false);
                    });
                }
            }else{
                if(this.editor){
                    //this.editor.fieldElements:获取到容器中实现的编辑组件
                    this.editor.fieldElements.forEach(function(node){
                        node.setCanEdit(true);
                    });
                }
            }
        },
        setValue:function(value){
            // summary:
            //      设置组件的值
            // params:
            //      value:组件的值
            // tags:
            //      private
            this.value = value||{};
            if(this.editor){
                this.editor.setData(this.value);
            }
        },
        getValue:function(){
            // summary:
            //      获取组件的值
            // tags:
            //      private
            return this.value||{};
        },
        // 功能：更改对象fields
        // 传递参数有 fields（array）
        alterLayout:function(fields) {
			// 设置编辑属性清单
			this.fields = fields;
            this._initEditor();
        }
    });
});