define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-construct",
    "dojo/Deferred",
    "dijit/_WidgetBase",
    "dojo/store/JsonRest",
    "dijit/form/ComboBox"
],function(declare,lang,array,domConstruct,Deferred,_WidgetBase,JsonRest,ComboBox){
    return declare("cq.app.widget.web.edit.Ref",[_WidgetBase],{
        // summary: 
        //      定义了web端的引用组件，继承自_WidgetBase.
        
        // htmlTemp:String
        //      组件的DOM结构字符串,并传递组件的属性。
        // tags:
        //      private
        htmlTemp:'<div id="{id}" data-cq-field="{field}" class="cq-field-input cq-form-ref" data-dojo-type="cq/app/widget/web/edit/Ref" data-dojo-props="field:\'{field}\',pageSize:\'{pageSize}\',targetMethod:\'{targetMethod}\',searchAttr:\'{searchAttr}\',labelAttr:\'{labelAttr}\',dataProvider:\'{dataProvider}\',canEdit:{canEdit}"></div>',
        
        // field:String
        //      组件名称
        //      实际可能是该组件需要的某些字段、组件的组件名、与之联动的组件的组件名或字段名构成的一段字符串。
        field:"",
        
        // 查询属性名
        searchAttr:"",
        
        // 显示的属性名，属性名中间以逗号隔开
        labelAttr:"",
        
        dataProvider:"",
        
        targetMethod:"",
        
        //pageSize:String 
        //      分页大小
        pageSize:"30",
        // start:String
        //      起始页
        start:"0",
        
        // canEdit:boolean
        //      编辑状态，true为可编辑。
        canEdit:true,
        buildRendering:function(){
            // summary:
            //      初始化组件
            // tags:
            //      private
            this.inherited(arguments);
            this.containerNode = this.containerNode||this.domNode;
            this._initRef();
        },
        setCanEdit:function(canEdit){
            // summary:
            //      设置组件是否可编辑。
            // params:
            //      canEdit:编辑状态
            // tags:
            //      public
            this.canEdit = canEdit;
            if(this.canEdit == false){
                this.box.set("disabled", "disabled");
            }else{
                this.box.set("disabled", "");
            }
        },
        _initRef:function(){
            // summary:
            //      实现一个ComboBox，加载到数据并绑定在ComboBox上面，然后将这个comboBox绑定到组件上
            if(!this.box){
                this.box = new ComboBox({
                    searchAttr:this.searchAttr,
                    queryExpr:"${0}",
                    labelType:"html",
                    labelFunc:lang.hitch(this, function(item, store){
                        // 如果有多个属性需要显示
                        // 显示宽度问题
                        if(lang.isString(this.labelAttr)){
                            this.labelAttr = this.labelAttr.split(",");
                        }
                        return array.map(this.labelAttr, lang.hitch(this, function(fld, index){
                            return "<label style='display:inline-block;width:100px;overflow:hidden;margin-right:5px;height:15px'>" + (item[fld]||"") + "</label>";
                        })).join("");
                        // var label = item[this.labelAttr] + "";
                        // return label.toLowerCase();
                    }),
                    // onSearch:function(){
                    //    var items = arguments[0];
                    //    if(items&&items.items){
                    //        arguments[0] = items.items;
                    //    }
                    //},
                    autoComplete:false,
                    onChange:lang.hitch(this, function(newValue){
                        this.onFieldChanged(this, this.field, this.box.item);
                        this.setValue(this.box.item);
                    })
                });
                this.box.placeAt(this.containerNode);
                if(this.canEdit == false){
                    this.box.set("disabled", "disabled");
                }
                // 数据源
                if(this.dataProvider){
                    if(lang.isString(this.dataProvider)){
                        require([this.dataProvider], lang.hitch(this, function(dataProvider){
                            // 当数据提供者加载完成之后，调用targetMethod方法，获取url
                            this.dataProvider = dataProvider;
                            var url = this.dataProvider[this.targetMethod]();
                            this.store = new JsonRest({target:url});
                            this.store.query = lang.hitch(this, function(){
                                arguments[1].count = this.pageSize;
                                var result = JsonRest.prototype.query.apply(this.store,arguments);
                                result.then = result.then(function(data){
                                    return data.items||data;
                                }).then;
                                return result;
                            });
                            this.setQuery({start:this.start, count:this.pageSize});
                            this.box.set("store", this.store);
                        }));
                    }
                }
            }else{
                this.setValue(this.getValue());
            }
        },
        setQuery:function(query){
            // summary：
            //      实现搜索功能
            // params:
            //      query:查询条件
            this.store.query(query, {
                start:this.start,
                count:this.pageSize
            });
        },
        onFieldChanged:function(widget, field, value){
            // summary:
            //      本对象发生变化后通知其他对象刷新的接口
            // params:
            //      widget:发生变化对象所在的view
            //      field:发生变化的对象
            //      value:改变后的值
            // tags:
            //      interface
        },
        setValue:function(value){
            // summary:
            //      设置组件的值
            // params:
            //      value:组件的值
            // tags:
            //      private
            this.value = value||{};
            if(this.box&&this.value&&this.value[this.labelAttr]){
                this.box.set("item", this.value);
            }
        },
        getValue:function(){
            // summary:
            //      获取组件的值
            // tags:
            //      private
            return this.value||{};
        }
    });
});