define([
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/json",
    "dojo/_base/lang",
    "cq/biz/object",
    "cq/biz/user/user",
    "cq/util/config/config",
    "dojo/Deferred",
    "dojo/dom-construct",
    "cq/util/sys/userAgent"
],function(declare, array, json, lang, object,user,config,Deferred,domConstruct, userAgent){
    var paymentQrDiv = null;
    var O = declare("", object.Schema, {
        modalName:"ts/order_h",
        //通过Id查询订单
        loadById:function(id){
            return this.execInCurrentShop("load_by_id",{_id:id});
        },
        //提交订单
        //参数：hash，属性和含义
        //  收货人和收货地址相关属性
        //  mids:购物车返回的mids数组，包含mid和数量，要购买的商品信息
        //  addressId:收货地址id
        //      没有传时按照之前的方式工作
        //      传了地址之后按照商家设置的商品运费计算
        createOrder:function(params,shopId){
            //params{dAddress,dTel,dName,remark,mids:[{mid:"",qty:""}]}
            params = lang.clone(params);
            params.mids = json.toJson(params.mids);
            params.shopId=shopId?shopId:this.getShopId();
            return this.exec("create_order",params);
        },
        //提交订单，处理了分销订单，以后逐步使用本接口，启用前一个接口
        //参数：hash，属性和含义
        //  收货人和收货地址相关属性
        //  mids:购物车返回的mids数组，包含mid和数量，要购买的商品信息
        //  addressId:收货地址id
        //      没有传时按照之前的方式工作
        //      传了地址之后按照商家设置的商品运费计算
        //  coupon要使用的优惠券，object，key为对应商家id，value为对应商家使用的优惠券id
        createDistributionOrder:function(params,shopId){
            //params{dAddress,dTel,dName,remark,mids:[{mid:"",qty:""}]}
            params = lang.clone(params);
            params.mids = json.toJson(params.mids);
            params.shopId=shopId?shopId:this.getShopId();
            //本订单使用的优惠券
            //params.coupon = JSON.stringify(params.coupon||{});
            return this.exec("create_distribution_order",params);
        },
        _getTAmount:function(params){
            params = lang.clone(params);
            params.mids = json.toJson(params.mids);
            return this.exec("get_tamount",params);
        },
        //参数：hash，属性和含义
        //  mids:购物车返回的mids数组，包含mid和数量，要购买的商品信息
        //  addressId:收货地址id
        //      没有传时按照之前的方式工作
        //      传了地址之后按照商家设置的商品运费计算
        getTAmount:function(params){
            //params{mids:[{mid:"",qty:""}]}
            if(!params.shopId) {
                params.shopId = this.getShopId();
            }
            // params.shopId = this.getShopId();
            return this._getTAmount(params);
        },
        //获取运费和优惠券信息
        //  {data:{item:{shopid1:{tAmount:10,totalSale:1000,shopInfo:{},matInfo:{},coupons:[]}}}}
        getCouponsAndTAmount:function(params){
            params.withCoupons = true;
            return this.getTAmount(params);
        },
        //通过ID查询订单详情
        loadDetailById:function(id){
            return this.execInCurrentShop("load_detail_by_id",{_id:id});
        },
         //查询当前
        loadDetail:function(params){
            params.userId = user.getCurrentUserId();
            return this.execInCurrentShop("load_detail",params);
        },
        //根据优惠价格(discountAmount)修改价格
        updateDetailDiscountAmount:function(params){
            return this.execInCurrentShop("update_discount_amount",params);
        },
        //保存上传凭证生成自动签收的定时任务
        saveAutomaticReceipt:function(params){
            params.platformShopId = config.getPlatformShopId();
            return this.execInCurrentShop("save_automatic_receipt",params);
        },
        loadWithDetails:function(params){
            params.userId = user.getCurrentUserId();
            return this.execInCurrentShop("load_with_details",params);
        },
        //查询本商家订单，可传不同的查询条件
        loadShopDetail:function(id){
            return this.getBaseUrl()+"/load_detail?shopId="+this.getShopId();
        },
        //根据商家加载订单 的url
        getShopWithDetailsUrl:function(){
            return this.getBaseUrl()+"/load_with_details?shopId="+this.getShopId()+"&fromShopId="+this.getShopId();
        },
        getWithDetailsUrl:function(){
            return this.getBaseUrl()+"/load_with_details?shopId="+this.getShopId()+"&userId="+user.getCurrentUserId();
        },
        //查询当前用户在本商家的订单，可传不同的查询条件
        getDetailUrl:function(){
            return this.getBaseUrl()+"/load_detail?shopId="+this.getShopId()+"&userId="+user.getCurrentUserId();
        },
        //查询当前用户所有订单状态的总数量
        getDetailCount:function(){
            return this.execInCurrentShop("load_detail_count",{userId:user.getCurrentUserId()});
        },
        //取消整张订单
        onCancelAll:function(id){
            return this.execInCurrentShop("cancel_by_id",{_id:id});
        },
        //标记已支付
        //setPayed:function(params){
        //    return this.execInCurrentShop("pay_complete",params);
        //},
        //订单审核
        setDetailAccepted:function(id){
            return this.execInCurrentShop("set_accepted_by_detailid",{detailId:id});
        },
        // 整单审核
        setAccepted:function(id){
            return this.execInCurrentShop("accept_by_id",{
                _id:id
            });
        },
        //订单发货
        setDetailDelivered:function(id){
            return this.execInCurrentShop("set_delivered_by_detailid",{detailId:id});
        },
        // 整单发货
        setDelivered:function(id){
            return this.execInCurrentShop("deliver_by_id",{
                _id:id,
                platformShopId: config.getPlatformShopId()
            });
        },
        //订单取消（明细）
        setDetailCanceled:function(id){
            return this.execInCurrentShop("set_canceled_by_id",{detailId:id});
        },
        // 整单取消
        setCanceled:function(id){
            return this.execInCurrentShop("cancel_by_id",{
                _id:id
            });
        },
        //评价
        setDetailEvaluated:function(id){
            return this.execInCurrentShop("set_evaluated_by_detailid",{detailId:id});
        },
        //退款
        setDetailRefounded:function(id){
            return this.execInCurrentShop("set_refounded_by_detailid",{detailId:id});
        },
        //收货
        setDetailReceipteded:function(id){
            return this.execInCurrentShop("set_receipted_by_detailid",{detailId:id});
        },
        // 整单收货
        setReceipteded:function(id){
            return this.execInCurrentShop("receipt_by_id",{
                _id:id
            });
        },
        evaluedByDetailId:function(params){
            params.userId = user.getCurrentUserId();
            return this.execInCurrentShop("evalue_by_user_and_detail_id",params);
        },
        //重新计算订单的支付方案
        generatePaymentSchemeById:function(id){
            return this.execInCurrentShop("generate_payment_scheme_by_id",{
                _id:id
            });
        },
        //依据对应的商品查询订单明细
        loadDetailByFarmerId:function(params){
            if(!params){
                params = {};
                
            }
            params.shopId = this.getShopId();
            params.farmer = user.getCurrentUserId();
            return this.execInCurrentShop("load_detail_by_farmerId", params);
        },
        // 找人代付
        // @param _id 订单主表Id 
        // @param content 消息内容
        // @param userName 代付人的用户名
        findAnotherToPay:function(params){
            params.shopId = this.getShopId();
            return this.execInCurrentShop("find_another_to_pay", params);
        },
        onPayed:function(param){
            var def = new Deferred();
            // 张松   如果是APP中的微信支付，payType未wxApp  2016/10/21
            //若不是微信环境或者商家的是订阅号，通过微信扫码支付
            if("weixin"==param.payType&&(!window.WeixinJSBridge||1==g_sys_info.wxType||!g_sys_info.wxVerification)){
                if(userAgent.isApp()){
                    param.payType = "wxApp";
                }else{
                    param.payType = "wxNative";
                }
            }
            // 如果是支付宝支付，判断是否是APP支付，是否安装支付宝插件，如果没有则继续网页支付，否则试用插件支付
            if("aliwap" == param.payType){
                if(userAgent.isApp() && cordova.plugins.AliPay){
                    param.payType = "aliapp";
                }else{
                    param.payType = "aliwap";
                }
            }
            param.returnUrl = location.href;
            this.execInCurrentShop("get_pay_param",param).then(lang.hitch(this,function(result){
                if(result.data.ok){
                    this.invokePayParam(lang.mixin({},param,result.data.item),def,result);
                }else{
                    // result.msg 就是错误原因,不明白为什么要重复alert
                    // 刘轩 2016/7/25注释
                    //alert(result.err);
                    def.resolve(result);
                }
            }));
            return def;
        },
        //分销订单支付参数{ids:[],payType:""}
        onPayedByIds:function (param) {
            var def = new Deferred();
            //若不是微信环境或者商家的是订阅号，通过微信扫码支付
            if("weixin"==param.payType&&(!window.WeixinJSBridge||1==g_sys_info.wxType||!g_sys_info.wxVerification)){
                if(userAgent.isApp()){
                    param.payType = "wxApp";
                }else{
                    param.payType = "wxNative";
                }
            }
            // 如果是支付宝支付，判断是否是APP支付，是否安装支付宝插件，如果没有则继续网页支付，否则试用插件支付
            if("aliwap" == param.payType){
                if(userAgent.isApp() && cordova.plugins.AliPay){
                    param.payType = "aliapp";
                }else{
                    param.payType = "aliwap";
                }
            }
            param.returnUrl = location.href;
            this.execInCurrentShop("get_pay_param_by_ids",param).then(lang.hitch(this,function(result){
                if(result.data.ok){
                    this.invokePayParam(lang.mixin({},param,result.data.item),def,result);
                }else{
                    // 统一由前台提示错误信息
                    //alert(result.err);
                    def.resolve(result);
                }
            }));
            return def;
        },
        invokePayParam:function(params,def,result){
            if("weixin"==params.payType){
                //微信网页支付
                WeixinJSBridge.invoke("getBrandWCPayRequest",params,function(res){
                    def.resolve(result);
                });
            }else if("wxNative"==params.payType){
                //微信扫码支付
                this._payWithWeiXinQr(params,def,result);
            }else if("point"==params.payType){
                def.resolve(result);
            }else if("wxApp"==params.payType){
                // 微信APP支付
                var payParams = {
                    partnerid: params.partnerid,
                    prepayid: params.prepayid,
                    noncestr: params.noncestr,
                    timestamp: params.timestamp,
                    sign: params.paySign,
                };
                Wechat.sendPaymentRequest(payParams, function () {
                    alert("支付成功");
                    def.resolve(result);
                }, function (reason) {
                    alert("支付失败: " + reason);
                    def.resolve(result);
                });
            }else if("bocNB2B" == params.payType || "bocB2CMobile" == params.payType){
                var form = domConstruct.create("form", {
                    method:"post",
                    action:params.action
                });
                for(var key in params.body){
                    domConstruct.create("input", {
                        type:"hidden",
                        name:key,
                        value:params.body[key]
                    }, form);
                }
                var subBtn = domConstruct.create("input", {
                    type:"submit",
                    value:"submit"
                }, form);
                subBtn.click();
                def.resolve(result);
            }else{
                if(params.payType==="aliwap"){
                    this._payWithAliWapParam(params,def,result);
                }else{
                    var payInfo = decodeURIComponent(params.payQueryString);
                    cordova.plugins.AliPay.pay(payInfo,function success(e){
                        def.resolve(result);
                    },function error(e){
                        alert("支付失败: " + e.memo);
                        def.resolve(result);
                    });
                }
            }
        },
        _payWithWeiXinQr:function(params,def,result){
            //生成二维码
            require([
                "cq/util/image/qrCode"
            ],function(qrCode){
                if(paymentQrDiv&&paymentQrDiv.parentNode){
                    paymentQrDiv.parentNode.removeChild(paymentQrDiv);
                }
                var div1 = paymentQrDiv = domConstruct.create("div",{
                    "class":"paymentQrDiv"
                },document.body),img1 = domConstruct.create("div",{
                    innerHTML:window.WeixinJSBridge?"请长按二维码支付喔~~":"微信扫一扫支付",
                    style:"font-size:20px;color:green;margin-top: 3%;"
                },div1);
                img1 = domConstruct.create("img",{
                    "class":"paymentQrImg",//action
                    src:qrCode.generatePNG(params.code_url,{
                        modulesize: 5,
                        margin: 2,
                        ecclevel: "M"
                    })
                },div1);
                img1 = domConstruct.create("span",{
                    innerHTML:"完成",
                    "class":"paymentQrBtn"
                },div1);
                img1.onclick = function(){
                    div1.parentElement.removeChild(div1);
                    div1 = null;
                    def.resolve(result);
                };
                img1 = domConstruct.create("span",{
                    innerHTML:"取消",
                    "class":"paymentQrBtn"
                },div1);
                img1.onclick = function(){
                    div1.parentElement.removeChild(div1);
                    div1 = null;
                    def.resolve(result);
                };
            });
        },
        _payWithAliWapParam:function(params,def,result){
            if(params.payUrl){
                if(window.WeixinJSBridge){
                    return window._AP?_AP.pay(params.payUrl):domConstruct.create("script",{
                        type:"text/javascript",
                        src:"/static/alipay/ap.js",
                        onload:function(){
                            _AP.pay(params.payUrl);
                        }
                    },document.body);
                }else if(userAgent.isMobile()){
                    if(window.cordova){
                        var ref = window.open(params.payUrl,"_blank");
                        ref.addEventListener("exit",function(){
							def.resolve(result);
                            ref.removeEventListener("exit",function(){});
						});
                        return;
                    }
                    require(["cq/util/edit/confirmFrame"],function(frame){
                        frame.show(params.payUrl).then(function(){
                            def.resolve(result);
                        });
                    });
                }else{
                    window.location.href = params.payUrl;
                    //window.open(params.payUrl);
                }
            }else{
                def.resolve(result);
            }
        },
        
        
        //B1B订单的相关接口
        //依据商品加载需要的运费
        getB2BTAmount:function(params){
            params.shopId = this.getShopId();
            return this._getTAmount(params);
        },
        getB2BCouponsAndTAmount:function(params){
            params.withCoupons = true;
            return this.getB2BTAmount(params);
        },
        //提交订单
        //参数同提交分销订单，参数和返回值参考createDistributionOrder
        createB2BOrder:function(params){
            //params{dAddress,dTel,dName,remark,mids:[{mid:"",qty:""}]}
            params = lang.clone(params);
            params.mids = json.toJson(params.mids);
            params.shopId = this.getShopId();
            //本订单使用的优惠券
            //params.coupon = JSON.stringify(params.coupon||{});
            return this.exec("create_distribution_order",params);
        },
        //支付订单
        //参数通支付分销订单
        onB2BPayByIds:function (param) {
            var def = new Deferred();
            //若不是微信环境或者商家的是订阅号，通过微信扫码支付
            if("weixin"==param.payType&&(!window.WeixinJSBridge||1==g_sys_info.wxType||!g_sys_info.wxVerification)){
                if(userAgent.isApp()){
                    param.payType = "wxApp";
                }else{
                    param.payType = "wxNative";
                }
            }
            // 如果是支付宝支付，判断是否是APP支付，是否安装支付宝插件，如果没有则继续网页支付，否则试用插件支付
            if("aliwap" == param.payType){
                if(userAgent.isApp() && cordova.plugins.AliPay){
                    param.payType = "aliapp";
                }else{
                    param.payType = "aliwap";
                }
            }
            // if("string"!=typeof(param.ids)){
                // param.ids = JSON.stringify(param.ids);
            // }
            if(!param.shopId) {
                param.shopId = this.getShopId();
            }            
            param.returnUrl = location.href;
            this.exec("get_pay_param_by_ids",param).then(lang.hitch(this,function(result){
                if(result.data.ok){
                    this.invokePayParam(lang.mixin({},param,result.data.item),def,result);
                }else{
                    def.resolve(result);
                }
            }));
            return def;
        },
        //取消整张订单
        onCancelB2BAll:function(id){
            return this.exec("cancel_by_id",{
                _id:id,
                shopId:this.getShopId()
            });
        },
        //确认收货
        setB2BDetailReceipteded:function(id){
            return this.execIn("set_receipted_by_detailid",{
                detailId:id,
                shopId:this.getShopId()
            });
        },
        //确认收货
        setB2BReceipteded:function(id){
            return this.execIn("receipt_by_id",{
                detailId:id,
                shopId:this.getShopId()
            });
        },
        //评价订单
        evaluedB2BByDetailId:function(params){
            params.userId = user.getCurrentUserId();            
            return this.execInCurrentShop("evalue_by_user_and_detail_id",params);
        },
        //查询订单，获取的参数和结果与之前的相同
        getB1BWithDetailsUrl:function(){
            return this.getBaseUrl()+"/load_with_details?shopId="+this.getShopId()+"&userId="+user.getCurrentUserId();
        },
        //通过Id查询订单
        loadB2BById:function(id){
            return this.exec("load_by_id",{
                _id:id,
                shopId:this.getShopId()
            });
        },
        //通过订单主表查询订单明细
        loadB2BDetailById:function(id){
            return this.exec("load_detail_by_id",{
                _id:id,
                shopId:this.getShopId()
            });
        },
        getB2BDetailCount:function(){
            return this.exec("load_detail_count",{
                shopId:this.getShopId(),
                userId:user.getCurrentUserId()
            });
        },
        summaryPayedCount:function(params){
            return this.exec("summary_payed_count",{shopId:this.getShopId(),fromShopId:this.getShopId(),days:params.days});
        },
        loadDetailShopById:function(params){
            return this.exec("load_detail_by_id",{
                _id:params.id,
                shopId:params.shopId
            });
        },
        loadShopById:function(params){
            return this.exec("load_by_id",{
                _id:params.id,
                shopId:params.shopId
            });
        }
    });
    var o = new O();
    lang.setObject("cq.biz.ts.order", o);
    o.Schema = O;
    return o;
});
