﻿define([
    "dojo/_base/lang",
    "dojo/Deferred",
    "../BoBase",
    "cq/biz/mat/mat",
    "cq/biz/mat/group",
    "cq/biz/shop/shop",
    "cq/biz/ts/cart",
    "cq/util/sys/query"
], function(lang, Deferred, BoBase, mat, group, shop, cart, query) {
    var matBo = new BoBase({
        getShopMatUrl: function() {
            return mat.getUrlByShop();
        },
        getMatCard: function(data) {
            return {
                icon: data.logo,
                label: "<div>" + data.mName + "</div>" +
                    "<div>￥" + data.retailPrice + "</div>" +
                    "<div>库存：" + data.qty + "</div><div>销量：" + data.totalSale + "</div>",
                data: data
            };
        },
        getMatCardForPurchase: function(data) {
            return {
                icon: data.logo,
                label: "<div>" + data.mName + "</div><div>￥" + data.purchasePrice + "</div>",
                data: data
            };
        },
        getMatQuery: function(cqRefInfo) {
            var params = { shopId: mat.getShopId(), qty: 1, isUse: true };
            var value = "";
            value = query.getQueryValue(cqRefInfo.search);
            if (value) {
                params.search = value;
            }
            value = query.getQueryValue(cqRefInfo.groupId);
            if (value) {
                params.groupId = value;
            }
            value = query.getQueryValue(cqRefInfo.orderBy);
            if (value) {
                params.orderBy = value;
            }
            return params;
        },
        getGroupQuery: function(cqRefInfo) {
            var params = { isUse: true,shopId:group.getShopId() };
            return params;
        },
        getOtherShopMatUrl: function() {
            return mat.getUrlByShop();
        },
        getMyShopMatQueryForPurchase: function(cqRefInfo) {
            var params = { shopId: mat.getShopId(), isPurchase: true };
            return params;
        },
        getOtherShopMatQuery: function(cqRefInfo) {
            var shopId = cqRefInfo && cqRefInfo.cqViewParam ? cqRefInfo.cqViewParam.params : mat.getShopId();
            var params = { shopId: shopId, qty: 1, isUse: true };
            return params;
        },
        onMatListClick: function(data, action) {
            window.moveToView('matDetail', data._id + "_" + (data.fromShopId && data.fromShopId._id));
        },
        loadOneMat: function(param) {
            var def = new Deferred();
            var m = param.cqViewParam.params.split("_");
            var mId = m[0];
            var shopId = m[1] ? m[1] : mat.getShopId();
            mat.loadById(mId, shopId).then(lang.hitch(this, function(data) {
                // 获取来源商家的信息
                shop.loadById(shopId).then(lang.hitch(this, function(shopData) {
                    data.data.item.supplierName = shopData.data.shopName;
                    data.data.item.tranRegion = shopData.data.tranRegion || "全国";
                    data.data.item.fromShopId = shopId;
                    def.resolve(data.data.item);
                }));
            }));
            return def;
        },
        getDetailTemplateForSale: function() {
            return "<div style='display:inline-block;'>" +
                "<img src='{logo}' style='width:380px;'/></div>" +
                "<div style='width:580px;float:right;'>" +
                "<h4>{mName}</h4>" +
                "<p><span class='label'>价格</span>￥{retailPrice}</p>" +
                "<p><span class='label'>规格</span>{sizeType}</p>" +
                "<p><span class='label'>库存</span>{qty}</p>" +
                "<p><span class='label'>销售区域</span>{tranRegion}</p>" +
                "<p><span class='label'>起订数量</span>{minQty}</p>" +
                "<p><span class='label'>供应商</span><a href='#supplyBus-{fromShopId}'>{supplierName}</a></p>" +
                "<p><span class='label'>评价</span>({commentNum})<span class='label'>平均得分</span>{scoreAvg}</p>" +
                "<p><div class='button' onclick='alert(\"待实现\");'>立即购买</div>" +
                "<div class='button' onclick='cq.biz.ts.cart.add(\"{_id}\",{minQty});window.moveToView(\"orderCar\")'>加入购物车</div></p>" +
                "</div><div>{content}</div>";
        },//cq.biz.ts.cart.add(\"{_id}\",{minQty});window.moveToView(\"orderInfo\");window.moveToView(\"orderCar\");
        getMatGroupUrl: function() {
            return group.getUrlByGroup();
        },
        getMatGroupCard: function(data) {
            return {
                icon: data.logo,
                label: data.gName,
                data: data
            };
        },
        onMatGroupListClick: function(data, action) {
            window.moveToView('matGroup', data._id);
        },
        fields: {
            mName: {
                type: "text",
                label: "商品名称",
				className:"middle-width",
                editable: true
            },
            code: {
                type: "text",
                label: "编码",
				className:"small-width",
                editable: true
            },
            sortName: {
                type: "text",
                label: "标准分类",
				className:"middle-width",
                editable: true
            },
            sortId: {
                type: "ref",
                label: "品类",
                dataProvider: "cq/biz/mat/sort",
                targetMethod: "loadNoSub",
                searchField: "sortName",
                labelFields: "sortName",
				className:"middle-width",
                editable: true
            },
            groupId: {
                type: "ref",
                label: "分组",
                dataProvider: "cq/biz/mat/group",
                targetMethod: "getGroupUrl",
                searchField: "gName",
                labelFields: "gName",
				className:"middle-width",
                editable: true
            },
            sizeType: {
                type: "text",
                label: "规格",
				className:"small-width",
                editable: true
            },
            retailPrice: {
                type: "text",
                label: "价格",
				className:"small-width",
                editable: true
            },
            logo: {
                type: "image",
                label: "首图"
            },
            images: {
                type: "images",
                label: "首图"
            },
            marketPrice: {
                type: "text",
                label: "市场价"
            },
            qty: {
                type: "text",
				className:"small-width",
                label: "库存"
            },
            totalSale: {
                type: "text",
                label: "销量"
            },
            minQty: {
                type: "text",
                label: "最小起订量"
            },
            freight: {
                type: "text",
                label: "运费"
            },
            freightTplId: {
                type: "ref",
                label: "运费模板",
                dataProvider: "cq/biz/shop/freightTemplate",
                targetMethod: "getLoadUrl",
                searchField: "title",
                labelFields: "title"
            },
            content: {
                type: "richText",
                label: "详细信息",
            },
            purchaseQty: {
                type: "text",
                label: "采购起订量"
            },
            purchasePrice: {
                type: "text",
                label: "采购价"
            },
            isPurchase: {
                type: "checkbox",
                label: "是否采购"
            },
            matSave: {
                type: "button",
                label: "确定",
                action: "onMatSave"
            },
            groupLogo: {
                type: "image",
                label: "品类图标",
                field: "logo"
            },
            gName: {
                type: 'text',
                label: "品类名称",
            },
            isUse: {
                type: "checkbox",
                label: "使用"
            },
            groupSave: {
                type: "button",
                label: "确定",
                action: "onGroupSave"
            },
            listActions:{
				type:"text",
				label:"",
				formatter:function(action,data){
					var v = "<div class='button' cq-action='delete'>删除</div><div class='button' onclick='window.moveToView(\"mat\",\""+data._id+"\");'>编辑</div>";
					return v;
				},
				className:"middle-width"
            },
            groupListActions:{
				type:"text",
				label:"",
				formatter:function(action,data){
					var v = "<div class='button' cq-action='delete'>删除</div><div class='button' onclick='window.moveToView(\"matGroup\",\""+data._id+"\");'>编辑</div>";
					return v;
				},
				className:"middle-width"
            }
        },
        // 添加商品
        getMatLayout: function() {
            if (!this.matLayout) {
                //店铺认证
                this.matLayout = [{
                    type: "form",
                    fields: this.getFormItems(["images", "mName", "code", "sortId", "groupId", "sizeType", "content", "retailPrice", "marketPrice", "qty", "totalSale", "minQty", "freight", "freightTplId", "purchaseQty", "purchasePrice", "isPurchase", "matSave"])
                }];
            }
            return this.matLayout;
        },
        // 商品在GRID中的布局
        getMatGridLayout: function() {
            if (!this.matGridLayout) {
                //店铺认证
                this.matGridLayout = this.getFormItems(["mName", "code","groupId", "sizeType", "retailPrice", "qty","listActions"]);
            }
            return this.matGridLayout;
        },
        // 初始化商品数据
        loadMatData: function(params) {
            var def = new Deferred();
            if (params) {
                mat.loadById(params.cqViewParam.params).then(function(data) {
                    if (data && data.data && data.data.ok === 1) {
						if(!data.data.item.images || data.data.item.images.length==0) data.data.item.images=[data.data.item.logo];
						if(data.data.item.sortId) data.data.item.sortId={_id:data.data.item.sortId,sortName:data.data.item.sortName};
                        def.resolve(data.data.item);
                    } else {
                        def.resolve({
                            isPurchase: true,
                            totalSale: 0,
                            qty: 9999
                        });
                    }
                });
            } else {
                def.resolve({
                    isPurchase: true,
                    totalSale: 0,
                    qty: 9999
                });
            }
            return def;
        },
        //验证畜牧商品辅助资料检查的接口
        verifyXmAssist: function() {},
        // 保存商品
        onMatSave: function(data) {
            // 保存数据
            var m = data;
            // 保存时对标准分类 初始化 后面组件做好后删除
            var iconItems = m.images;
            //检查是否设置了主图
            if (iconItems.length > 0) {
                m.logo = iconItems[0];
            }
            // 优先检测code 避免发出请求
            if (!m.code) {
                alert("没有填写编码");
                return;
            }
            //检查编码重复性
            mat.checkCodeRepeat(m.code).then(lang.hitch(this, function(data) {
                if (data && data.data && data.data.ok === 1) {
                    var dataCode = "";
                    if (data.data.item) {
                        dataCode = data.data.item;
                    }
                    if (dataCode) {
                        //判断有异常，执行free操作
                        alert("商品编码重复，请重新填写");
                        return;
                    }
                    // 检查必填项是否已填
                    var msg = this.verify(m);
                    if (msg) {
                        //判断有异常，执行free操作
                        alert(msg);
                        return;
                    }
                    //检查是否设置了细节图
                    var detailImages = [];
                    if (iconItems.length > 2) {
                        for (var i = 1; i < iconItems.length - 1; i++) {
                            detailImages.push(iconItems[i].icon);
                        }
                    }
                    m.images = detailImages;
                    this.saveMat(m);
                } else {
                    alert(data.data.msg);
                }
            }));

        },
        verify: function(data) {
            var msg = "";
            if (!data.logo) {
                msg = "必须设置主图";
            } else if (!data.code) {
                msg = "商品编码必须填写";
            } else if (!data.mName) {
                msg = "商品名称必须填写";
            } else if (!data.sizeType) {
                msg = "规格必须填写";
            } else if (!data.retailPrice) {
                msg = "价格必须填写";
            } else if (!data.content) { // 张松   添加商品详情内容的审核   2016/06/20
                msg = "商品详情必须填写";
            } else if (!data.qty) {
                msg = "库存必须填写";
            } else {
                msg = this.verifyXmAssist(data);
            }
            return msg;
        },
        saveMat: function(m) {
            mat.save(m).then(function(data) {
                if (data && data.data && data.data.ok === 1) {
                    history.back();
                } else {
                    alert(data.data.message);
                }
            });
        },
        // 添加品类
        getGroupLayout: function() {
            if (!this.groupLayout) {
                //店铺认证
                this.groupLayout = [{
                    type: "form",
                    fields: this.getFormItems(["groupLogo", "code", "gName", "groupSave"])
                }];
            }
            return this.groupLayout;
        },
        getGroupGridLayout: function() {
            if (!this.groupGridLayout) {
                //店铺认证
                this.groupGridLayout = this.getFormItems(["code", "gName","groupListActions"]);
            }
            return this.groupGridLayout;
        },
		onGroupListAction:function(data,action,list){
			if(action == "delete"){
				if(!confirm("确认要删除吗？")) return;
				group.save({_id:data._id,isUse:false}).then(function(){list.removeItemByData(data);});
			}
		},
        // 初始化品类数据
        loadGroupData: function(params) {
            var def = new Deferred();
            if (params && params.cqViewParam.params) {
                group.loadById(params.cqViewParam.params).then(lang.hitch(this, function(data) {
                    if (data && data.data && data.data.ok === 1) {
                        // 显示信息
                        // 处理图片显示
                        var info = data.data.item;
                        info.groupLogo = info.logo;
                        def.resolve(info);
                    } else {
                        def.resolve({
                            isUse: true
                        });
                        alert(data.data.msg);
                    }
                }));
            } else {
                def.resolve({
                    isUse: true
                });
            }
            return def;
        },
        onGroupSave: function(params) {
            var m = params;
            var msg = this.checkFiled(m);
            if (msg) {
                //判断有异常，执行free操作
                alert(msg);
                return;
            }
            // 对上传图片的logo做处理
            m.logo = m.groupLogo;
            group.save(m).then(lang.hitch(this, function(data) {
                if (data && data.data && data.data.ok === 1) {
                    //保存后执行free操作
                    //window.moveToView("matGroupList");
                    history.back();
                } else {
                    alert(data.data.msg);
                }
            }));
        },
        checkFiled: function(data) {
            var msg = "";
            if (!data.gName) {
                msg = "品类名称还没有填写";
                return msg;
            } else if (!data.code) {
                msg = "品类编码还没有填写";
                return msg;
            }
        }
    });
    return matBo;
});
