define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojox/mobile/View",
    "dojox/mobile/EdgeToEdgeStoreList",
    "cq/ui/view/_ViewMixin",
    "dijit/registry",
    "cq/biz/article/articleGroup",
    "dojo/store/JsonRest"
], function (declare, lang, View, EdgeToEdgeStoreList, _ViewMixin, registry, articleGroup,JsonRest) {
    return declare("cq.ui.b.a.NewsGroupList", [View, _ViewMixin], {
        start : 0,
        pageSize:10,
        onAfterShow : function () {
            this.setQuery(this.storeList.get("query"), {
                start : 0,
                count : this.pageSize
            });
        },
        onFirstShow : function () {
            var option = {
                left : {
                    iconButton : ' ',
                    arrow : "left",
                    label : "返回",
                    onClick : function () {
                        history.back();
                    }
                },
                title : "资讯类别"
            };
            this.addHeader(option);
            option = {
                isHeader:true,
                placeHolder : "请输入资讯名称，按回车查询",
                type : "search",
                class : "cqSearchBox",
                onInput : lang.hitch(this, function (evt) {
                    if (evt.keyCode == 13) {
                        this.search = evt.target.value;
                        var condition = this.storeList.get("query");
                        condition.groupName = evt.target.value;
                        this.setQuery(condition);
                        evt.target.value = "";
                    }
                })
            };
            this.addTextBox(option);
            option = {
                columns : [{
                        label : '添加类别',
                        icon : 'images/icon/addnews.png',
                        iconClass: 'footer-tab-img',
                        onClick : function () {
                            window.moveToView("newsGroup");
                        }
                    }
                ],
                valueClass : 'font-size3',
                labelClass : 'font-size1',
                className : 'fix-footer',
                isFooter : true
            };
            this.addColumns(option);
            this.storeList = new EdgeToEdgeStoreList({
                append : true,
                constraint : false,
                query : {},
                class : "icon-list-item",
                variableHeight : true,
                onComplete : lang.hitch(this, function (data) {
                    for (var i = 0; i < data.length; i++) {
                        data[i]._label = "<div class='mat-list-mname mat-list-margin'>" + data[i].groupName + "</div>";
                        data[i]._logo = data[i].logo ? data[i].logo : 'images/nomat.png';
                        data[i].moveTo = "#";
                    }
                    this.storeList.generateList(data);
                    this.end = this.start + data.length - 1;
                    this.busy = false;
                }),
                queryOptions : {
                    count : this.pageSize,
                    start : this.start
                },
                itemMap : {
                    _label : "label",
                    _logo : "icon",
                    moveTo : "moveTo"
                }
            });
            // 创建数据源
            this.store = new JsonRest({
                target : articleGroup.getLoadUrl()
            });
            // 将数据源赋值给storeList
            this.onLoadListData(this.storeList.setStore(this.store,{},{
                start:0,
                count:this.pageSize
            }),this.start);
            this.storeList.placeAt(this.centerPanel.containerNode, "last");
            this.storeList.on("click", lang.hitch(this, this.onClickList));
            this.centerPanel.on("afterscroll", lang.hitch(this, this.onLoadMore));
        },
        onClickList:function(evt){
            // 得到点击的ListItem
            var li = registry.getEnclosingWidget(evt.target);
            if (li.declaredClass != "dojox.mobile.ListItem")
                return;
            var _id = li.params._id;
            window.moveToView("newsGroup", _id);
        },
        onLoadMore : function (evt) {
            //判断是否正在加载，判断是否向上拖动
            if (!this.busy&&evt.afterBottom) {
                this.start = this.start + this.pageSize;
                if(this.canLoadMore(this.start)){
                    console.log("加载更多类别");
                    this.busy = true;
                    setTimeout(lang.hitch(this, function () {
                        var queryResult = this.storeList.setQuery(this.storeList.get("query"), {
                            start : this.start,
                            count : this.pageSize
                        });
                        this.onLoadListData(queryResult,this.start,this.pageSize);
                    }), 1000);
                }
            }
        },
        setQuery : function (query) {
            this.start = 0;
            this.storeList.destroyDescendants();
            var queryResult = this.storeList.setQuery(query, {
                start : this.start,
                count : this.pageSize
            });
            this.onLoadListData(queryResult,this.start,this.pageSize);
        }
    });
});