define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/store/JsonRest",
    "dojo/on",
    "dojo/dom-class",
    "dojox/mobile/EdgeToEdgeStoreList",
    "dojox/mobile/View",
    "dijit/registry",
    "cq/ui/view/_ViewMixin",
    "cq/biz/im/questionMsg",
	"cq/util/sys/date"
], function(declare, lang, JsonRest, on, domClass, EdgeToEdgeStoreList, View, registry, _ViewMixin, questionMsg, date) {
    return declare("cq.ui.b.im.QuestionMsgList", [View, _ViewMixin], {
        // 初始化开始值
        start: 0,
        // 设置每次加载的条数
        pageSize: 15,
        // 设置查询条件
        setQuery: function(query) {
            this.start = 0;
            this.storeList.destroyDescendants();
            this.onLoadListData(this.storeList.setQuery(query, {
                start: this.start,
                count: this.pageSize
            }), this.start, this.pageSize);
        },
        // 初始化相关接口
        onAfterShow: function() {
            if (this.storeList.store) {
                this.setQuery(this.storeList.get("query"));
            } else {
                this.onLoadListData(this.storeList.setStore(this.store, this.params, {
                    start: 0,
                    count: this.pageSize
                }), this.start, this.pageSize);
            }
            this.refreshData();
        },

        onFirstShow: function() {
            var option = {
                left: {
                    iconButton: ' ',
                    arrow: "left",
                    label: "返回",
                    onClick: function() {
                        history.back();
                    }
                },
                title: "问题咨询"
            };
            this.addHeader(option);
			//添加顶部搜索栏
            option = {
                placeHolder: "请输入关键字，按回车进行查询",
                type: "search",
                class: "cqSearchBox",
                isHeader: true,
                onInput: lang.hitch(this,function(evt){
                    if(evt.keyCode==13){
                        this.search = evt.target.value;
                        var condition = this.storeList.get('query');
                        condition.search = this.search;
                        this.setQuery(condition);
                        evt.target.value = "";
                    }
                })
            };
			this.addTextBox(option);
            this.storeList = new EdgeToEdgeStoreList({
                append: true,
                constraint: false,
                query: {},
                "class": "icon-list-item",
                variableHeight: true,
                onComplete: lang.hitch(this, function(data) {
                    for (var i = 0; i < data.length; i++) {
                        data[i]._label = "<div class='webkit-list'>"
                            +"<div class='user-list-left user-list-margin'>提出日期:"+(date.formatISODate(data[i].pDate) || '')+"</div>"
                            +"</div>"
                            +"<div class='webkit-list'>"
                            +"<div class='user-list-right user-list-margin'>姓名:"+(data[i].name ||'')+"</div>"
							+"<div class='user-list-left user-list-margin'>电话:"+(data[i].phone || '')+"</div>"
                            +"</div>"
                            +"<div class='webkit-list'>"
                            +"<div class='user-list-right user-list-margin'>问题:"+(data[i].title ||'')+"</div>"
                            +"</div>"
                            +"<div class='forth-btn'>"
                            +"<span class='deletequestion float-right'>删除</span>"
                            +"<span class='editquestion float-right'>编辑</span>"
                            +"</div>";
                        data[i]._logo='images/userIcon.png';
                    }
                    this.storeList.generateList(data);
                    this.busy = false;
                }),
                // 查询条件
                queryOptions: {
                    count: this.pageSize,
                    start: this.start
                },
                itemMap: { _label: "label", moveTo: "moveTo",_logo: "icon" }
            });
            // 创建数据源
            this.params = {};
            this.store = new JsonRest({ target: questionMsg.getLoadUrl() });
            this.storeList.placeAt(this.centerPanel.containerNode, "last");
            this.storeList.on("click", lang.hitch(this, this.onClickList));
            on(this.centerPanel, "afterscroll", lang.hitch(this, this.onLoadMore));

            // 添加底部
            option = {
                columns: [{
                    label: '添加问题',
                    icon: 'images/icon/crmicon.png',
                    iconClass: 'footer-tab-img',
                    onClick: function() {
                        window.moveToView("questionMsgDetail");
                    }
                }],
                valueClass: 'font-size3',
                labelClass: 'font-size1',
                className: 'fix-footer',
                isFooter: true
            };
            this.addColumns(option);
            this.onAfterShow();
        },

        // 重写拖拽加载更多事件
        onLoadMore: function(evt) {
            if (!this.busy && evt.afterBottom) {
                this.start = this.start + this.pageSize;
                if (this.canLoadMore(this.pageSize)) {
                    this.busy = true;
                    setTimeout(lang.hitch(this, function() {
                        this.onLoadListData(this.storeList.setQuery(this.storeList.get("query"), {
                            start: this.start,
                            count: this.pageSize
                        }), this.start, this.pageSize);
                    }), 1000);
                }
            }
        },

        // 重写List的点击事件
        onClickList: function(evt) {
            // 得到点击的ListItem
            var li = registry.getEnclosingWidget(evt.target);
            if (li.declaredClass != "dojox.mobile.ListItem") return;
            var dom = evt.target;
            var _id = li.params._id;
            //编辑
            if (dom && domClass.contains(dom, "editquestion")) {
                window.moveToView("questionMsgDetail", _id);
            }
            //删除
            if (dom && domClass.contains(dom, "deletequestion")) {
                var isDelete = confirm("确认删除此问题?");
                if (isDelete) {
                    questionMsg.deleteById(_id).then(lang.hitch(this, function(data) {
                        if (data.data.ok === 1) {
                            alert("删除成功");
                            this.onAfterShow();
                        } else {
                            alert("删除失败");
                        }
                    }));
                }
            }
        }
    });
});
