﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/store/JsonRest",
    "dojo/dom-class",
    "dojo/touch",
    "dojo/query",
    "dojox/mobile/View",
    "dojox/mobile/EdgeToEdgeStoreList",
    "cq/ui/view/_ViewMixin",
    "cq/biz/im/channelMsg",
    "cq/biz/blob/blob",
    "cq/biz/user/user",
    "cq/widgets/item/LiveListItem"
],function( declare, lang, JsonRest, domClass, touch, query, View, EdgeToEdgeStoreList, _ViewMixin, channelMsg, blob, user, LiveListItem){
    return declare("cq.ui.common.im.Channel",[View,_ViewMixin],{
        pageSize:20,
        onFirstShow : function(){
            //创建页面基本元素，初始化界面元素
            //创建头部
            var option = {
                left: {
                    iconButton: ' ',
                    arrow: 'left',
                    label: '返回',
                    onClick: function(){
                        history.back();
                    }
                },
                title:'直播详情'
            };
            this.addHeader(option);
            option = {
                isHeader: true,
                columns:[{
                    label:"直播区",
                    onClick:function(e){
                        this.setChannelSelected(e.target);
                        this.toLiveView();
                    }
                },{
                    label:"讨论区",
                    onClick:function(e){
                        this.setChannelSelected(e.target);
                        this.toDiscussView();
                    }
                }
                ],
                labelClass:'font-size1 tab-selector'
            };
            this.tabSelector= this.addColumns(option);
            this.setChannelSelected(query('.tab-selector',this.tabSelector)[0]);
            //头部选择区域，并监听点击事件显示相应的界面
            //初始化直播视图
            this.liveView = new View();
            //指定加载数据的位置
            this.liveView.placeAt(this.centerPanel.containerNode, "last");
            //初始化讨论视图
            this.discussView = new View();
            //指定加载数据的位置
            this.discussView.placeAt(this.centerPanel.containerNode, "last");
            //直播区的列表显示
            this.liveList= new EdgeToEdgeStoreList({
                append: true,
                constraint: false,
                query:{},
                class:"icon-list-item",
                variableHeight:true,
                itemRenderer:LiveListItem,
                queryOptions: {
                    count: this.pageSize,
                    start: this.liveStart
                },
                itemMap: { _label: "label", moveTo: "moveTo" }
            });
            this.liveList.placeAt(this.liveView, "last");
            //讨论区的列表显示
            this.discussList= new EdgeToEdgeStoreList({
                append: true,
                constraint: false,
                query:{},
                class:"icon-list-item",
                variableHeight:true,
                itemRenderer:LiveListItem,
                queryOptions: {
                    count: this.pageSize,
                    start: this.liveStart
                },
                itemMap: { _label: "label", moveTo: "moveTo" }
            });
            this.discussList.placeAt(this.discussView, "last");
            //为中间视图区域绑定滚动事件和点击事件
            this.centerPanel.on("afterscroll", lang.hitch(this, this.onLoadMore));
            this.centerPanel.on("click", lang.hitch(this, this.onListClick));
            //添加底部输入框和传图片的栏目
            
            //侦听发送图片和发送文本的事件
            option = {
                placeHolder: "请输入回复内容",
                class: "cqSearchBox",
                type: "search",
                onInput: lang.hitch(this, this.onMessageEnter)
            };
            this.textBox = this.addTextBox(option);
            this.footerColumn = this.addColumns({
                forceWidth: false,
                isFooter: true,
                columns: [{
                    className: "flex1",
                    label: ""
                }, {
                    className: "miniIcon imageSelector",
                    label: ""
                }]
            });
            this.textBox.placeAt(query(".flex1", this.footerColumn)[0], "only");
            
            //发图片消息
            this.own(touch.release(query(".miniIcon", this.footerColumn)[0], lang.hitch(this, function() {
                blob.selectAndUpload().then(lang.hitch(this, this.onUploadImage));
            })));
            //侦听收到新消息的事件，显示到对应的列表中
            this.connect(channelMsg,"onDiscussMsg",function(msg){
                this.discussList.addChild(new LiveListItem({params:msg}),0);
            });
            this.connect(channelMsg,"onLiveMsg",function(msg){
                this.liveList.addChild(new LiveListItem({params:msg}),0);
            });
            
            //进入频道
            this.onAfterShow();
        },
        getCurrentUserInfo:function(){
            return user.getUserInfo();
        },
        getChannelId:function(){
            return window.did;
        },
        //在onafterShow里面根据数据来刷新页面或者更新显示
        onAfterShow : function(){
            //获取当前用户的信息
            this.getCurrentUserInfo().then(lang.hitch(this, function(userData) {
                this.currentUserInfo = userData && userData.data && userData.data.item || {};
            }));
            //切换到当前频道
            this.toChannel(this.getChannelId());
            //切换到直播室界面
            this.toLiveView();
        },
        toChannel:function(channelId){
            this.channelId = channelId;
            channelMsg.join(channelId);
            this.liveList.destroyDescendants();
            this.discussList.destroyDescendants();
            this.loadNextPage("liveList");
            this.loadNextPage("discussList");
        },
        //当前显示的消息区域
        currentMode:"liveList",
        toLiveView:function(){
            this.currentMode = "liveList";
            this.liveView.show();
            this.centerPanel.scrollTo({x:0,y:0});
        },
        toDiscussView:function(){
            this.currentMode = "discussList";
            this.discussView.show();
            this.centerPanel.scrollTo({x:0,y:0});
        },
        //离开当前页面时离开房间
        onBeforeTransitionOut:function(){
            this.inherited(arguments);
            channelMsg.leave(this.channelId);
        },
        onLoadMore: function(e){
            if(!this.busy&&e.afterBottom){
                this.busy = true;
                this.loadNextPage(this.currentMode);
            }
        },
        loadNextPage:function(listType){
            var list = this[listType],queryInfo = null;
            if(list.store){
                queryInfo = list.get("query");
                queryInfo.channelId = this.channelId;
                list.setQuery(queryInfo,{
                    start:list.getChildren().length,
                    count:this.pageSize
                }).then(lang.hitch(this,function(){
                    this.busy = false;
                }));
            }else{
                var store = new JsonRest({
                    target:channelMsg.getLoadUrl()
                });
                queryInfo = listType==="liveList"?{
                    channelId:this.channelId,isHost:true
                }:{
                    channelId:this.channelId
                };
                list.setStore(store,queryInfo,{
                    start:0,count:this.pageSize
                });
            }
        },
        setChannelSelected: function(evt){
            //清空所有节点选中样式
            var items = query('.tab-selector',this.domNode);
            for(var i = 0;i<items.length;i++){
                domClass.remove(items[i],"selected");
            }
            domClass.add(evt,"selected");
        },
        onListClick : function(evt){
            if(evt.target.localName == "img"){
                if(domClass.contains(evt.target.parentNode, "richText")){
                    var imgSrcUri = evt.target.src;
                    this.showImg(imgSrcUri); 
                } 
            }
        },
        getMsg:function(msg){
            return msg;
        },
        onUploadImage: function(imageInfo){
            if (imageInfo && imageInfo.items && imageInfo.items[0]) {
                channelMsg.save(this.getMsg({
                    senderId: this.currentUserInfo._id,
                    channelId: window.did,
                    sendNick: this.currentUserInfo.pName || this.currentUserInfo.nickName || this.currentUserInfo.wxNick || "",
                    sendLogo: this.currentUserInfo.logo || null,
                    sendTime: new Date(),
                    msgType: 1,
                    msgImage: imageInfo.items[0].uri,
                    isHost: true
                }));
            }
        },
        onMessageEnter: function(evt) {
            if (evt.keyCode == 13) {
                this.search = evt.target.value;
                if (!this.search) {
                    return;
                }
                channelMsg.save(this.getMsg({
                    senderId: this.currentUserInfo._id,
                    channelId: window.did,
                    sendNick: this.currentUserInfo.pName || this.currentUserInfo.nickName || this.currentUserInfo.wxNick || "",
                    sendLogo: this.currentUserInfo.logo || null,
                    sendTime: new Date(),
                    msgType: 0,
                    msgText: this.search,
                    isHost: true
                }));
                evt.target.value = "";
            }
        },
        
    });
});