define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-class",
    "dojo/query",
    "dijit/registry",
    "dojox/mobile/EdgeToEdgeList",
    "dojox/mobile/EdgeToEdgeStoreList",
    "dojox/mobile/ListItem",
    "cq/biz/blob/blob",
    "cq/biz/ts/pm/project",
    "cq/util/sys/date"
],function(declare,lang,array,domClass,query,registry,EdgeToEdgeList,EdgeToEdgeStoreList,ListItem,blob,project,date){
    return declare("cq.ui.b.p._PrjMixin",null,{
        // 只读界面的处理
        // 公共方法
        projectStateLabels:{
        },
        // 获取状态显示标签
        getProjectStateLabel:function(state){
            return this.projectStateLabels[state]||"";
        },
        // 浏览界面和列表界面显示日期的特殊处理
        decorateLabelData:function(projectItem){
            projectItem.startDate = projectItem.startDate?date.formatISODate(projectItem.startDate):"";
            projectItem.endDate = projectItem.endDate?date.formatISODate(projectItem.endDate):"";
            projectItem.startReviewDate = projectItem.startReviewDate?date.formatISODate(projectItem.startReviewDate):"";
            projectItem.endReviewDate = projectItem.endReviewDate?date.formatISODate(projectItem.endReviewDate):"";
            //商家名称
            projectItem.fromShopName = projectItem.fromShopId&&projectItem.fromShopId.shopName||"";
            //项目名称
            projectItem.srcProjectName = projectItem.srcProjectId&&projectItem.srcProjectId.title||"";
            //申请时间
            projectItem.createAt = date.formatISODate(projectItem.createAt);
            return projectItem;
        },
        // 列表界面特有功能
        listMoveView:"",
        // 数据加载开始下标
        start:0,
        // 每次加载数据的条数
        pageSize : 10,
        // 添加列表界面的顶部标题栏
        addListTitleHeader:function(title,right){
            var option = {
                left : {
                    iconButton : ' ',
                    arrow : "left",
                    label : "返回",
                    onClick : function() {
                        history.back();
						this.isEditing = false;
                    }
                },
                title:title||"",
                right:right
            };
            this.addHeader(option);
        },
        // 初始化查询结果列表
        initStoreList:function(){
            this.storeList=new EdgeToEdgeStoreList({
                append: true,
                constraint: false,
                query:{},
                "class":"icon-list-item",
                variableHeight:true,
                //获取数据完成后调用的的处理方法
                onComplete:lang.hitch(this,function(data){
                    array.forEach(data,this.initListItemLabel,this);
                    this.storeList.generateList(data);
                    this.end=this.start+data.length-1;
                    this.busy = false;
                }),
                queryOptions: {
                    count: this.pageSize,
                    start: this.start
                },
                itemMap:{
                    _label:"label",
                    moveTo:"moveTo"
                }
            });
            this.storeList.placeAt(this.centerPanel.containerNode,"last");
            this.storeList.on("click",lang.hitch(this,this.onClickList));
            this.centerPanel.on("afterscroll",lang.hitch(this,this.onLoadMore));
        },
        // 查询结果的点击事件处理函数
        onClickList:function(evt){
            // 得到点击的ListItem
            var li = registry.getEnclosingWidget(evt.target);
            if(li.declaredClass!="dojox.mobile.ListItem") return;
            var _id = li.params._id;
            window.moveToView(this.listMoveView,_id);
        },
        // 加载更多的事件处理
        onLoadMore:function(evt){
            if(this.busy||(0<this.end&&(this.start+this.pageSize-1>this.end))){
                return true;
            }
            if(evt.afterBottom){
                this.start = this.end+1;
                this.busy = true;
                setTimeout(lang.hitch(this,function(){
                    this.storeList.setQuery(this.storeList.get("query"), {
                        start: this.start,
                        count: this.pageSize
                    }).then(lang.hitch(this,function(){
                        this.centerPanel.resize();
                    }));
                }), 1000);
            }
        },
        //列表界面的列表内容
        initListItemLabel:function(projectItem){
            this.decorateLabelData(projectItem);
            projectItem.moveTo="#";
            projectItem._label = "<div class='mat-list-mname mat-list-margin'>项目名称："+(projectItem.title||"")+
                "</div><div class='mat-list-mname mat-list-margin'>开始时间："+projectItem.startDate+
                "</div><div class='mat-list-mname mat-list-margin'>结束时间："+projectItem.endDate+
                "</div><div class='mat-list-qty mat-list-margin'>状态："+this.getProjectStateLabel(projectItem.state)+
                "</div>";
        },
        //浏览界面特有功能
        initAttachList:function(data){
            this.attachmentLists.destroyDescendants();
            //创建附件lists
            var items=data.attachs||[];
            for (var i=0;i<items.length;i++) {
                this.attachmentLists.addChild(new ListItem({
                    label:this.getAttachLabel(items[i]),
                    variableHeight:true
                }));
            }
        },
        hasBlank:true,
        //获取浏览界面附件列表的内容
        getAttachLabel:function(projectItem){
            return "<div class='forth-btn'>"+
			(this.hasBlank?"<span><a href='"+projectItem.blankUrl+"' downLoad='"+projectItem.title+"'>下载空表文件</a></span>":"")+
                "<span><a href='"+projectItem.sampleUrl+"' downLoad='"+projectItem.title0+"'>下载样表文件</a></span></div>";
        },
        // 刷新操作按钮区域
        // 浏览界面状态和对应状态下按钮的显示隐藏状态
        dtlOpers:{},
        // 依据数据获取当前页面的状态
        getDtlState:function(){
            var data = this.getData();
            return data&&data.state||0;
        },
        // 依据当前数据刷新界面按钮的显示状态
        refreshOperColumn:function(){
            var dtlOpers = this.dtlOpers[this.getDtlState()];
            if(dtlOpers&&dtlOpers.length){
                array.forEach(this.dtlOperColumnNode.children,function(node,index){
                    domClass[dtlOpers[index]?"remove":"add"](node,"none");
                });
                domClass.remove(this.dtlOperColumnNode,"none");
            }
            this.resize();
        },
        // 隐藏界面操作按钮
        hideOperColumn:function(){
            domClass.add(this.dtlOperColumnNode,"none");
        },
        // 刷新界面数据显示
        refreshProjectData:function(){
            this.hideOperColumn();
            //获取平台数据，参数：项目Id，如何获得项目Id
            project.loadById(window.did).then(lang.hitch(this,function(data){
                var projectItem = data.data&&data.data.item||{};
                projectItem = this.decorateLabelData(projectItem);
                //通过ID获取商铺名称:未实现
                this.setData(projectItem);
                this.refreshData();
                this.initAttachList(projectItem);
                this.refreshOperColumn(projectItem);
            }));
        },
        createDtlAttachList:function(){
            //参数option：元素属性
            this.attachmentLists=this.addLists(option={
                class:'attachmentLists',
                variableHeight:true,
                //宽度拉伸
                isEdge:true,
                lists:[]
            });
        },
        createDtlOperColunm:function(options){
            this.dtlOperColumnNode = this.addColumns(options);
            this.hideOperColumn();
            return this.dtlOperColumnNode;
        },
        // 可编辑界面的处理
        // 附件列表
        attachmentList:null,
        // 添加附件到附件列表
        addAttachment:function(attach,deleteable,canEditName){
            attach = attach||{
                title:"",
                title0:"",				
                blankUrl:"",
                sampleUrl:""
            };
            var attachItem = new ListItem({
                label:"<div><span>空表文件名称："+(
                        false!==canEditName?"<input class='attachmentName' type='text' value=''" + (attach.title||"") + "'/>":(attach.title||"")
                    )+"</span></div>"+
					"<div><span>样表文件名称："+(
                        false!==canEditName?"<input class='attachmentName0' type='text' value=''" + (attach.title0||"") + "'/>":(attach.title0||"")
                    )+"</span></div>"+(
                        false!==canEditName?"空表文件：<a href='javascript:void(0);' class='blankUrl file'>选择空表文件<input type='file' class='empty'></a><span class='emptyValue' value=''></span></div>":""
                    )+"<div>"+(
                        false!==canEditName?"样表文件":"样表文件"
                    )+"：<a href='javascript:void(0);' class='sampleUrl file'>选择样表文件<input type='file' class='sample'></a><span class='sampleValue' value=''></span></div>",
                rightText:deleteable?"<div><span class='attachment-cancel'>删除</span></div>":"",
                variableHeight:true,
                params:attach
            });
            this.attachmentList.addChild(attachItem);
            this.initAttachItemEvent(attachItem);
        },
        // 添加附件时，附件对象的事件处理
        initAttachItemEvent:function(attachItem){
            if(0<attachItem.get("rightText").indexOf("attachment-cancel")){
                // 删除按钮
                attachItem.connect(query(".attachment-cancel",attachItem.domNode)[0],"onclick",function(e){
                    this.destroyRecursive();
                });
            }
            if(query(".attachmentName",attachItem.domNode).length){
                // 标题编辑框
                attachItem.connect(query(".attachmentName",attachItem.domNode)[0],"onblur",function(e){
                    this.params.title = e.target.value;
                });
            }
            if(query(".attachmentName0",attachItem.domNode).length){
                // 标题编辑框
                attachItem.connect(query(".attachmentName0",attachItem.domNode)[0],"onblur",function(e){
                    this.params.title0 = e.target.value;
                });
            }			
            if(query(".empty",attachItem.domNode).length){
                // 空表文件
                attachItem.connect(query(".empty",attachItem.domNode)[0],"onchange",function(e){
                    blob.upload(e.target).then(lang.hitch(this,function(data){
                        this.params.blankUrl = data.items[0].uri;
                    }));
                });
            }
            // 样表文件
            attachItem.connect(query(".sample",attachItem.domNode)[0],"onchange",function(e){
                blob.upload(e.target).then(lang.hitch(this,function(data){
                    this.params.sampleUrl = data.items[0].uri;
                }));
            });
        },
        // 依据附件列表获取附件数据
        getAttachsFromList:function(){
            return array.map(this.attachmentList.getChildren(),function(attachItem){
                return attachItem.params;
            });
        },
        createEdtAttachList:function(){
            this.attachmentList=new EdgeToEdgeList();
            this.attachmentList.placeAt(this.centerPanel.containerNode,"last");
        }
    });
});