/**
 * 创建人：杜飞军
 * 创建时间：2016/08/10
 * 创建内容：
 *     显示模板标题、运送到、费率
 */
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-class",
    "dojo/store/JsonRest",
    "dijit/registry",
    "dojox/mobile/View",
    "dojox/mobile/EdgeToEdgeStoreList",
    "cq/ui/view/_ViewMixin",
    "cq/biz/shop/freightTemplate"
], function(declare, lang, array, domClass, JsonRest, registry, View, EdgeToEdgeStoreList, _ViewMixin, freightTemplate) {
    return declare('cq.ui.b.s.FreightList', [View, _ViewMixin], {

        //为列表设置的数据开始默认值
        start: 0,
        pageSize: 20,
        // 每次进入页面重新加载数据
        onAfterShow: function() {
            if (!this.store) {
                this.store = new JsonRest({
                    target: freightTemplate.getLoadUrl()
                });
                // 将数据源赋值给storeList
                var queryResult = this.storeList.setStore(this.store, this.getQuery(), {
                    start: this.start,
                    count: this.pageSize
                });
                this.onLoadListData(queryResult, this.start, this.pageSize);
            } else {
                this.setQuery();
            }
        },
        onFirstShow: function() {
            var option = {
                left: {
                    iconButton: ' ',
                    arrow: "left",
                    label: "返回",
                    onClick: function() {
                        history.back();
                    }
                },
                title: "管理运费"
            };
            this.addHeader(option);
            this.storeList = new EdgeToEdgeStoreList({
                append: true,
                constraint: false,
                query: {},
                class: "icon-list-item",
                variableHeight: true,
                onComplete: lang.hitch(this, function(data) {
                    for (var i = 0; i < data.length; i++) {
                        data[i]._label = "<div class='mat-list-mname mat-list-margin'>" + data[i].title + "</div>";
                        data[i]._label += this.getFreightLabel(data[i]);
                        data[i]._label += array.map(data[i].exceptions, this.getFreightLabel, this).join("");
                        data[i]._label += "<div class='forth-btn'><span class='deletequater float-right chatbtn'>删除</span></div>";
                        data[i].moveTo = "#";
                    }
                    this.storeList.generateList(data);
                    this.end = this.start + data.length - 1;
                    this.busy = false;
                }),
                queryOptions: {
                    count: this.pageSize,
                    start: this.start
                },
                itemMap: { _label: "label", logo: "icon", moveTo: "moveTo" }
            });

            //置于地点
            this.storeList.placeAt(this.centerPanel.containerNode, "last");
            //绑定点击事件
            this.storeList.on("click", lang.hitch(this, this.onClickList));
            //绑定事件--加载更多
            this.centerPanel.on("afterscroll", lang.hitch(this, this.onLoadMore));

            // 底部添加按钮
            option = {
                columns: [{
                    label: "添加运费模板",
                    className: 'flex2 footer-tab-highlight noverify',
                    onClick: function() {
                        window.moveToView('freightDetail');
                    }
                }],
                forceWidth: false,
                isFooter: true
            };
            this.addColumns(option);
            this.onAfterShow();
        },
        getFreightLabel: function(data) {
            return "<div class='mat-list-margin margin-top10' style='word-break: break-all;overflow: hidden;text-overflow: ellipsis;display:-webkit-box;-webkit-line-clamp: 2;-webkit-box-orient: vertical;'>" + (data.provinceId ? (
                    data.provinceId.distName + (
                        data.cityId ? (
                            "," + data.cityId.distName + (
                                data.countyId ? "," + data.countyId.distName : ""
                            )
                        ) : ""
                    )
                ) : "全国") + ":" + data.minQty + "件以内 " + window.fixedTo(data.minFee, 3) + "元；每增加" + data.incQty + "件，增加运费" + window.fixedTo(data.incFee, 3) + "元" +
                "</div>";
        },

        // 获取查询条件
        getQuery: function() {
            return {};
        },

        //点击事件，点击下拉列表节点项所触发事件触发函数，参数evt，返回值：无
        onClickList: function(evt) {
            // 得到点击的ListItem
            var dom = evt.target;
            var li = registry.getEnclosingWidget(evt.target);
            //删除
            if (dom && domClass.contains(dom, "deletequater")) {
                var isDelete = confirm("确认删除这条信息！");
                if (isDelete) {
                    var _id = li.params._id;
                    freightTemplate.deleteById(_id).then(lang.hitch(this, function(data) {
                        if (data.data.ok == 1) {
                            alert("删除成功");
                            this.onAfterShow();
                        } else {
                            alert(data.data.message);
                        }
                    }));
                }
                return;
            }
            if (li.declaredClass != "dojox.mobile.ListItem") {
                return;
            }
            window.moveToView("freightDetail", li.params._id);
        },

        // 重新赋值列表并刷新显示
        setQuery: function() {
            this.start = 0;
            this.end = -1;
            this.storeList.destroyDescendants();
            // 徐阿雪 修改加载问题 20160623
            var queryResult = this.storeList.setQuery(this.getQuery(), {
                start: this.start,
                count: this.pageSize
            });
            this.onLoadListData(queryResult, this.start, this.pageSize);
        },

        //加载更多，参数evt，返回值：无
        onLoadMore: function(evt) {
            if (!this.busy && evt.afterBottom) {
                this.start = this.start + this.pageSize;
                if (this.canLoadMore(this.start)) {
                    this.busy = true;
                    setTimeout(lang.hitch(this, function() {
                        var queryResult = this.storeList.setQuery(this.storeList.get("query"), {
                            start: this.start,
                            count: this.pageSize
                        });
                        this.onLoadListData(queryResult, this.start, this.pageSize);
                    }), 300);
                }
            }
        },
    });
});
