/*
 *  修改人: 杜飞军
 *  修改时间: 2016-06-22
 *  修改内容:
 *  修改登陆失败返回的值显示
 *  用户登录后 保存用户名用于下次登陆时能够显示
 *  存储在localStarge中  key : userName
 *  在手机端获得文本节点焦点
 *  登陆完成后是否保存密码  key : userPass
 *  增加找回密码功能
 *  增加回车键登陆功能
 */
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/topic",
    "dojo/query",
    "dojo/on",
    "dojox/mobile/View",
    "cq/ui/view/_ViewMixin",
    "cq/biz/user/user"
], function(declare, lang, topic, query, on, View, _ViewMixin, user) {
    return declare("cq.ui.b.s.Login", [View, _ViewMixin], {
        destroyOnSvcChanged: false,
        // 进入页面后初始化页面数据
        onAfterShow: function() {
            // 获得存储的用户名
            this.localUserName = localStorage.getItem("userName");
            // 获得保存的密码
            this.localUserPass = localStorage.getItem("userPass");
            // 创建一个空对象保存本页面所需数据
            var loginData = {};
            // 判断存储的用户名是否存在
            if (this.localUserName) {
                // 若存在保存页面对象
                loginData.userName = this.localUserName;
            }
            // 检测是否保存了密码
            if (this.localUserPass) {
                loginData.password = this.localUserPass;
                loginData.userPass = true;
            }
            // 设置本页面所需数据
            this.setData(loginData);
            // 先阻止自动获得焦点
            this.userFocus.forEach(function(item) {
                item.blur();
            });

            this.onFoucsEntry();
            this.enter = on(window, "keydown", lang.hitch(this, this.onEnterLogin));
            // 刷新页面数据
            this.refreshData();
        },
        //初始化相关接口
        onFirstShow: function() {
            // setData 应该放在onAfterShow中
            // this.setData({});
            var option = {
                left: {
                    iconButton: ' ',
                    arrow: "left",
                    label: "返回",
                    onClick: function() {
                        this.enter.remove();
                        window.moveToView("main");
                    }
                },
                title: "登录",
                right: {
                    iconButton: ' ',
                    onClick: this.loginSubmit,
                    label: "确定"
                }
            };
            this.addHeader(option);
            option = {
                isEdge: true,
                lists: [{
                    label: '手机号',
                    noArrow: true,
                    edit: {
                        type: 'tel',
                        field: 'userName',
                        placeHolder: '请填写手机号',
                        pattern: "[0-9]*"
                    }
                }, {
                    label: '密码',
                    noArrow: true,
                    edit: {
                        type: 'password',
                        field: 'password',
                        placeHolder: '必须输入密码'
                    }
                }, {
                    label: '记住密码',
                    noArrow: true,
                    edit: {
                        type: 'checkbox',
                        field: 'userPass'
                    }
                }, {
                    className: "mblListItem btn-vcode",
                    rightText: '找回密码',
                    onClick: lang.hitch(this, function() {
                        this.enter.remove();
                        window.moveToView('retrieve');
                    })
                }],
                class: "top-bottom-margin16"
            };
            var loginButView = this.addLists(option);
            // 能在页面初始化后得到对象
            this.userNameBut = query("[__data_field='userName']", loginButView.domNode)[0];
            this.userPassBut = query("[__data_field='password']", loginButView.domNode)[0];
            this.userFocus = query("input", loginButView.domNode);
            setTimeout(lang.hitch(this, function() {
                query("[__data_field=userName]", this.domNode)[0].focus();
            }), 0);
            query("[__data_field=password]", this.domNode)[0].onkeydown = lang.hitch(this, function(event) {
                if (event.keyCode == 13) {
                    this.loginSubmit();
                }
                return true;
            });
            // 键盘事件
            on(this.userNameBut, "keydown", lang.hitch(this, this.onNumberOnly));
            this.onAfterShow();
        },
        // 只允许数字文本输入
        onNumberOnly: function(e) {
            // 默认阻止所有键盘值返回
            event.returnValue = false;
            // 如果是数字键盘则返回键盘值
            if ((48 <= e.keyCode && e.keyCode <= 57) || (96 <= e.keyCode && e.keyCode <= 105)) {
                event.returnValue = true;
            } else if (e.keyCode === 8 || e.keyCode === 46 || e.keyCode === 37 || e.keyCode === 39 || e.keyCode === 9) {
                // 允许键盘删除 tab 左右移动的操作
                event.returnValue = true;
            }
        },
        // 回车键登陆
        onEnterLogin: function(e) {
            if (13 === e.keyCode) {
                this.loginSubmit();
            }
        },
        loginSubmit: function() {
            var msg = this.verify();
            var userData = this.getData();
            //手机号，密码都满足条件时验证登录
            if (msg.length === 0) {
                user.loginShop(userData.userName, userData.password).then(lang.hitch(this, function(success) {
                    if (success.ok === 1) {
                        // 完成后保存用户名
                        localStorage.setItem("userName", userData.userName);
                        // 完成后检测状态 判断是否存储密码
                        if (userData.userPass) {
                            localStorage.setItem("userPass", userData.password);
                        } else {
                            // 清除保存的密码
                            localStorage.removeItem("userPass");
                        }
                        query("[__data_field=password]", this.domNode)[0].value = "";
                        this.enter.remove();
                        window.moveToView(user.nextPage || 'shopList', "auto");
                        topic.publish("/cqthink/app/login/loginSuccess");
                    } else {
                        alert(success.msg || "无效的用户名或密码");
                    }
                }));
            } else {
                //手机号，密码有为空的时候
                alert(msg);
            }
        },
        //验证手机号，密码是否为空
        verify: function() {
            var msg = "";
            if (!this.getData().userName) {
                msg = "必须输入手机号";
                return msg;
            }
            if (!this.getData().password) {
                msg = "必须输入密码";
                return msg;
            }
            return msg;
        },

        // 检测是否保存过用户或密码 并给于相应的focus
        onFoucsEntry: function() {
            // 用户名和密码都没有  给于账户focus
            // 第一次加载会存在焦点错误 先给 延迟处理
            setTimeout(lang.hitch(this, function() {
                if (!this.localUserName && !this.localUserPass) {
                    this.userNameBut.focus();
                } else if (this.localUserName && !this.localUserPass) {
                    // 用户名存在密码没有 给于密码focus
                    this.userPassBut.focus();
                } else {
                    this.userFocus.forEach(function(item) {
                        item.blur();
                    });
                }
                // 都存在不用给focus
            }), 500);
        }
    });
});
