/**
    变更人员： 杜飞军
    变更内容： 代码规范，添加注释，文件迁移
    变更时间： 2016/05/17
 */
define([
    "dojo/_base/declare",
    "dojox/mobile/View",
    "dojo/_base/lang",
    "dojox/mobile/ListItem",
    "cq/biz/user/user",
    "cq/ui/view/_ViewMixin",
    "cq/util/edit/jsonRef",
    "dojox/mobile/EdgeToEdgeList",
    "dojo/_base/array",
    "dijit/registry",
    "cq/biz/user/care",
    "dojo/query",
    "dojox/mobile/_EditableListMixin"
], function (declare, View, lang, ListItem, user, _ViewMixin, jsonRef, EdgeToEdgeList, array, registry, care, query) {
    // module:
    //      cq/ui/b/a/CareNotice
    return declare("cq.ui.b.a.CareNotice", [View, _ViewMixin], {
        // 初始化开始值
        start : 0,
        // 初始化编辑状态
        isEditing:false,
        // 已选择的客户的id列表
        selectedUserIds:[],
        // 监听删除的对应项
        onDeleteItem:function(item){
            //依据item找到对应列表项
            var id = item.params._id,
            userId = item.params.userId._id;
            //删除数据库中对应数据
            care.delete(id).then(lang.hitch(this,function(){
                // 删除后更新数据
                this.onRefCareList();
            }));
            //销毁item本身
            item.destroyRecursive();
        },
        // 监听List文本框 失去焦点处理
        onListBlur:function(e){
            //若目标为文本框
            if(e.target.type == "text"){
                //依据文本框找到对应列表项——registry.getEnclosingWidget
                var careRemark = registry.getEnclosingWidget(e.target);
                //若文本框中的内容和对应列表项的remark不同
                if(careRemark.params.remark != e.target.value){
                    //修改对应列表项的remark并保存到数据库
                    var data = {
                        "_id":careRemark.params._id,
                        "remark": e.target.value
                    };
                    care.save(data).then(lang.hitch(this,this.onRefCareList));
                }
            }
        },
        // 监听List拖动 交换位置处理
        onMoveItem:function(item,startIndex,endIndex){
            //若起止位置不同
            if(startIndex!= endIndex){
                //找到列表中的所有列表项
                var cares = array.map(this.careList.getChildren(),function(item,index){
                    return {
                        _id:item.params._id,
                        cOrder:index+1
                    };
                });
                //保存当前顺序，之后刷新当前界面
                care.updateCaresOrder(cares).then(lang.hitch(this,this.onRefCareList));
            }
        },
        //添加关怀榜列表项的方法
        onAddCare:function(){
            //选人
            var params = {
                    target:user.getUserUrl(),
                    query:{},
                    searchField:"pName",labelField:"pName"
                };
                //这里存在BUG 在选中人的时候存在多一个列表
                jsonRef.startEdit("", this, params).then(lang.hitch(this,function(data){
                //若选到了人，依据选中的人，保存列表项
                if(data&&data._id){
                    if(0===this.selectedUserIds.length||false===array.some(this.selectedUserIds,function(userId){
                        return userId==data._id;
                    })){
                        care.save({
                            userId:data._id,
                            cOrder:this.careList.domNode.childElementCount+1,
                            remark:data.pName
                         }).then(lang.hitch(this,this.onRefCareList));
                    }
                }
            }));
        },
        // 初始化焦点对像
        blurHandlers:[],
        // 数据刷新处理
        onRefCareList : function(){
            // 加载关怀榜数据(cq/biz/user/care 的方法)——异步操作
            // 清理列表中已有列表项
            this.careList.destroyDescendants();
            // 遍历所有数据，添加到列表中
            care.load().then(lang.hitch(this,function(data){
                // 移除监控
                while(this.blurHandlers.length){
                    this.blurHandlers.pop().remove();
                }
                data = data.data.items;
                // 初始化存储用户ID
                this.selectedUserIds = [];
                for(var e = 0; e < data.length; e++){
                      data[e].remark = data[e].remark ? data[e].remark : "";
                      var  label = '<div class="" style="display:inline-block;width:33%"><span>'+(e+1)+'</span>'+
                                  '</div><div class="" style="display:inline-block;width:33%"><span>'+data[e].userId.pName+'</span>'+
                                  '</div><input style="display:inline-block;width:33%" type="text" class="" value='+ data[e].remark +'>';
                      // 存储用户的ID 用于检测是否已经存在
                      this.selectedUserIds.push(data[e].userId._id);
                      var careItem = new ListItem({label : label,params : data[e]});
                      this.careList.addChild(careItem);
                }
                query("input",this.careList.domNode).forEach(function(node){
                    this.blurHandlers.push(this.connect(node,"onblur",this.onListBlur));
                },this);
                // this.connect(query("input",this.careList.domNode),"onblur",this.onListBlur);
                this.careList.startEdit();
            }));
        },
        // 进入界面之后的操作，刷新界面显示的操作
        onAfterShow:function(){
            // 执行基类操作
            this.inherited(arguments);
            if(this.isEditing){
                return ;
            }else{
                this.isEditing = true;
                this.onRefCareList();
            }
        },
        // 初始化内容显示
        onFirstShow: function () {
            // 添加头部，返回按钮和标题
            var options = {
                left: {
                    iconButton: ' ',
                    arrow: "left",
                    label: "返回",
                    onClick: lang.hitch(this,function () {
                        this.isEditing = false;
                        //window.moveToView("home");
                        history.back();
                    })
                },
                title: "关怀榜",
            };
            this.addHeader(options);
            // 显示标题栏
            options = {
                isEdge : true,
                lists : [
                  {
                    label : '<div style="width:33%;display:inline-block"><span>排行</span>'+
                            '</div><div style="width:33%;display:inline-block"><span>姓名</span>'+
                            '</div><div style="width:33%;display:inline-block"><span>职务</span></div>',
                    noArrow : true,
                    edit : {type : 'text', field : '', placeHolder : '',pattern:""}
                  }
                ],
                class:""
            };
            this.addLists(options);
            // 添加可编辑的EdgeToEdgeList
            this.careList = new EdgeToEdgeList({
                "class":"icon-list-item",
                editable : true
            });
            this.careList.placeAt(this.careListScroll.containerNode,"last");
            // 侦听List的事件
            // 列表项删除图标被点击的事件——onDeleteItem，调用this.onDeleteItem
            this.connect(this.careList,"onDeleteItem",this.onDeleteItem);
            // 根节点的onblur事件，调用this.onListBlur
            // this.connect(this.careList.domNode,"onblur",this.onListBlur);
            // 列表项移动完成的时间——onMoveItem
            this.connect(this.careList,"onMoveItem",this.onMoveItem);
            // 添加底部添加按钮，点击之后调用this.onAddCare
            options = {
                columns:[{
                    label:'添加',
                    onClick:lang.hitch(this,this.onAddCare)
                }],
                labelClass:'',
                "className":'padding-top-bottom16',
                isFooter:true
            };
            this.addColumns(options);
            // 执行进入界面的操作
            this.onAfterShow();
        },
        // 初始化List滚动值
        indexElement : function(){
            this.careListScroll = this.centerPanel;
        }
    });
});