define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-class",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/Deferred",
    "dojo/store/JsonRest",
    "dojox/mobile/FixedSplitter",
    "dojox/mobile/Pane",
    "dojox/mobile/EdgeToEdgeStoreList",
    "cq/biz/im/msg",
    "cq/biz/im/msg_p",
    "cq/util/sys/date",
    "cq/biz/user/user",
    "cq/widgets/item/TalkListItem",
    "cq/biz/shop/shop"
],function(declare,lang,array,domClass,domConstruct,domStyle,Deferred,JsonRest,FixedSplitter,Pane,EdgeToEdgeStoreList,msg,msg_p,date,user,TalkListItem, shop){
    return declare("",[FixedSplitter],{
        pageSize:20,
        start:0,
        variablePane:1,
        orientation: "V",
        buildRendering:function(){
            this.inherited(arguments);
            this.topPane = new Pane({
                style:"height:42px;border-bottom:2px solid #999"
            });
            this.addChild(this.topPane);
            this.msgPane = new Pane();
            this.addChild(this.msgPane);
            this.msgPane.on("click",lang.hitch(this,this.onTalkListClick));
            
            //添加对话消息列表
            this.storeList = new EdgeToEdgeStoreList({
                baseClass:"",
                append : true,
                constraint: false,
                query:{},
                class:"icon-list-item talk-list",
                variableHeight:true,
                onComplete:lang.hitch(this,this.onMsgDataLoaded),
                // 查询条件
                queryOptions: {
                    count: this.pageSize,
                    start: this.start
                }
            });
            
            this.storeList.placeAt(this.msgPane.domNode);
            
            this.bottomPane = new Pane({
                style:"height:48px;border-top:2px solid #999"
            });
            this.addChild(this.bottomPane);
            this.textMsgInput = domConstruct.create("textarea",{
                style:"width:96%;display:block;margin:auto;"
            },this.bottomPane.domNode);
            this.connect(this.textMsgInput,"onkeydown",lang.hitch(this, this.onMessageEnter));
            //侦听msg中新消息的推送，并在列表中显示
            this.connect(msg,"onShowMsg",this.onNewMsg);
        },
        initTalk:function(targetUserId,targetShopId){
            // 设置toppane的innerHTML
            shop.loadById(targetShopId).then(lang.hitch(this, function(shopData){
                if(shopData){
                    this.topPane.domNode.innerHTML = shopData.data.shopName;
                }
                //从参数中读取对话双方和商家的id
                this.currentShopId = targetShopId;
                this.targetUserId = targetUserId;
                this.storeList.destroyDescendants();
                this.canScrollToBottom = true;
                user.getUserInfo().then(lang.hitch(this,function(data){
                    this.currentUserInfo = data&&data.data&&data.data.item;
                    this.onLoadMore({
                        beforeTop:true
                    }).then(lang.hitch(this,this.scrollToBottom));
                }));
            }));
        },
        
        canScrollToBottom:true,
        scrollToBottom:function(){
            if(this.storeList.domNode.lastElementChild){
                this.storeList.domNode.lastElementChild.scrollIntoView();
            }
        },
        //加载对话消息，得到结果之后的处理函数
        onMsgDataLoaded:function(datas){
            //便利所有数据，向列表中添加第一个孩子
            array.forEach(datas,function(item){
                this.storeList.addChild(this.getListItemByData(item),0);
            },this);
            this.busy = false;
        },
        //依据一行数据得到一个消息列表项
        getListItemByData:function(data){
            //处理数据并传入talkListItem zkj
            var dataMsg = data&&data.data&&data.data.item||data;
            dataMsg.isHost = (dataMsg.senderId&&dataMsg.senderId._id)===this.currentUserInfo._id;
            dataMsg.createAt = dataMsg.createAt||date.nowISOString();
            return new TalkListItem({
                params:dataMsg
            });
        },
        //加载更多
        onLoadMore:function(e){
            var def = new Deferred();
            //若非busy且可以加载更多且从上往下拖动
            var start = this.storeList.getChildren().length;
            var queryInfo = null,queryResult = null;
            if(!this.busy){
                this.busy = true;
                queryInfo = {
                    shopId: this.currentShopId,
                    userId1: this.currentUserInfo._id,
                    userId2: this.targetUserId 
                };
                if(!this.storeList.store){
                    var store = new JsonRest({
                        target:msg.getSvcMsgUrl()
                    });
                    queryResult = this.storeList.setStore(store,queryInfo,{
                        start: start,
                        count: this.pageSize
                    });
                    queryResult.then(function(){
                        def.resolve();
                    });
                }else{
                    this.canScrollToBottom = false;
                    setTimeout(lang.hitch(this,function(){
                        queryResult = this.storeList.setQuery(queryInfo, {
                            start : start,
                            count : this.pageSize
                        });
                        queryResult.then(function(){
                            def.resolve();
                        });
                    }),1000);
                }
            }
            return def;
        },
        onTalkListClick:function(evt){
            //若点击的目标为图片且不是头像，看大图)
            if(evt.target.localName == "img"&&domClass.contains(evt.target.parentNode, "msgContent")){
                var imgSrcUri = evt.target.src;
                require([
                    "cq/widgets/bigImg"
                ],function(bigImg){
                    bigImg.show(url);
                });
            }
        },
        onMessageEnter:function(evt){
            //在列表最后添加自己发送的消息
            if(evt.keyCode==13){
                this.search = evt.target.value;
                if (!this.search) {
                    return;
                }
                //保存到服务器
                var data = {
                    shopId: this.currentShopId,
                    senderId: this.currentUserInfo._id,
                    receiverId: this.targetUserId,
                    msgContent: this.search,
                    msgType: "text"
                };
                msg.save(data);
                data.senderId = this.currentUserInfo;
                data.receiverId = {_id:this.targetUserId};
                evt.target.value = this.search = "";
                this.storeList.addChild(this.getListItemByData(data));
                this.scrollToBottom();
            }
        },
        onUploadImage:function(imageInfo){
            //得到结果之后，添加图片消息，上传图片消息
            if (imageInfo && imageInfo.items && imageInfo.items[0]) {
                var data = {
                    shopId: this.currentShopId,
                    senderId: this.currentUserInfo._id,
                    receiverId: this.targetUserId,
                    msgType: "image",
                    msgContent: imageInfo.items[0].uri,
                };
                msg.save(data);
                data.senderId = this.currentUserInfo;
                data.receiverId = {_id:this.targetUserId};
                this.storeList.addChild(this.getListItemByData(data));
                this.scrollToBottom();
            }
        },
        onNewMsg:function(msgData){
            var senderId = msgData.senderId;
            senderId = senderId&&senderId._id||senderId;
            //若接收人为自己，且发送人为对话目标人
            if(senderId==this.targetUserId){
                //设置消息为已读
                msgData.isUse = true;
                //添加列表项到列表的最后
                this.storeList.addChild(this.getListItemByData(msgData));
                this.scrollToBottom();
            }
        }
    });
});