﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-style",
    "dojo/Deferred",
    "dojox/mobile/View",
    "dojox/mobile/SpinWheel",
    "dojox/mobile/SpinWheelSlot",
    "cq/ui/view/_ViewMixin",
    "cq/util/edit/overlay"
],function(declare,lang,array,domStyle,Deferred,View,SpinWheel,SpinWheelSlot,_ViewMixin,overlay){
    return declare("",[View,_ViewMixin],{
        //滚轮配置项，数组，每个slot的宽度,id，数组，宽度
        slotProps:[],
        //所有的滚轮
        slots:null,
        //当前每列的数据
        slotValues:null,
        //私有的一步对象
        _selectDef:null,
        _slotStarted:false,
		//开始选择
        startSelect:function(params){
            //依据参数，初始化成slot需要的初始数据
            this.initSelect.apply(this,arguments);
            //固定高度，刚好是滚轮和确定取消按钮的高度
            domStyle.set(this.domNode, "height", "250px");
            //将选择器显示到overlay中
            overlay.showInOverlay(this.domNode);
            //若没有创建slot，创建slot，定时startup，并依据数据刷新slot显示
            if(!this._slotStarted){
                this.createSlots();
                this._slotStarted = true;
                setTimeout(lang.hitch(this,function(){
                    this.startup();
                    this.initSlotByData();
                }),300);
            }else{
                //直接语句数据刷新slot显示
                this.initSlotByData();
            }
            
            //创建并返回异步对象
            this._selectDef = new Deferred();
            return this._selectDef;
        },
        //依据数据初始化界面显示
        initSlotByData:function(){
            var initSlot = lang.hitch(this,function(slot){
                var def = null;
                if(0===slot.slotIndex){
                    def = this.loadRootData();
                }else{
                    def = this.loadDataByUpper(this._getDataKey(this.slotValues[slot.slotIndex-1]));
                }
                def.then(lang.hitch(this,function(items){
                    //依据加载好的数据，初始化slot的显示
                    this.initSlot(slot,items,this.slotValues[slot.slotIndex]||items[0],true);
                    this.onSlotInited(slot);
                    if(slot.nextSlot){
                        initSlot(slot.nextSlot);
                    }else{
                        this.resize();
                    }
                }));
            });
            initSlot(this.slots[0]);
        },
        _getDataKey:function(data){
            return "undefined"===typeof(data&&data._id)?data||"":data._id;
        },
        getDataKey:function(data){
            return this._getDataKey(data);
        },
        //初始化界面元素
        createSlots:function(){
            //创建滚轮
            this.spinwheel = new SpinWheel({
                style: "width:320px;",
                "class":"districtSpin"
            });
            this.spinwheel.placeAt(this.centerPanel.domNode, "before");
            //隐藏可以拖动的区域
            domStyle.set(this.centerPanel.domNode, "display", "none");
            //创建确定取消按钮
            this.addColumns({
                columns: [{
                    label: "确认",
                    className: 'flex1 footer-tab-dark h-center',
                    onClick: function() {
                        this.selectFinished();
                    }
                }, {
                    label: "取消",
                    className: 'flex1 footer-tab-highlight h-center',
                    onClick: function() {
                        this.selectCanceled();
                    }
                }],
                forceWidth: false,
                isFooter: true
            });
            this.slots = [];
            var slot = null;
            //创建每一个滚轮  slotProp结构obj{_id,label,upId,index,level},index 是哪个位置
            array.forEach(this.slotProps,function(slotProp,index){
                slot = this.createSlot(slot,slotProp,index);
                this.slots.push(slot);
            },this); 
        },
        //创建一个slot
        createSlot:function(upSlot,slotProp,index){
            //需要设置上级的下级，会影响到滚动完成之后的事件处理
            //创建slot
            var slot = new SpinWheelSlot({
                style: { textAlign: "center", width: slotProp }
            });
            //修改上级的属性
            //  若没有上级slot，为第一个
            //  若有上级，上级的下级是新创建的这个
            //  若没有下级，为最后一个
            if(upSlot){
                slot.upSlot = upSlot;
                upSlot.nextSlot = slot;
            }
            slot.slotIndex = index;
            
            this.spinwheel.addChild(slot);
            //绑定滚动完成之后的事件处理  
            this.connect(slot, "onFlickAnimationEnd", function() {
                this.setValueBySlot(slot);
                //若忽略本次滚动，忽略本次滚动，不能再忽略下次滚动
                if(slot.preventNext){
                    slot.preventNext = false;
                    return;
                }
                this.onSlotScrollEnd(slot);
            });
            return slot;
        },
        setValueBySlot:function(slot){
            var value = null,label = slot.get("value");
            array.some(slot.dataItem,function(item){
                if(item&&label===item.label){
                    value = item;
                    return true;
                }
            });
            this.slotValues[slot.slotIndex] = value;
        },
        //滚动完成之后的事件处理函数
        onSlotScrollEnd:function(slotObj){
            //若非最后一个，加载下一个滚轮的数据，并初始化
            if(slotObj.nextSlot){
                this.loadDataByUpper(this._getDataKey(this.slotValues[slotObj.slotIndex])).then(lang.hitch(this,function(items){
                    var slot = this.slots[slotObj.slotIndex+1];
                    //依据加载好的数据，初始化slot的显示
                    this.initSlot(slot,items,items[0],false);
                    //选中的是第一个，若选中之前就是选中的第一个，不会发生滚动事件。直接执行滚动完成的操作
                    if(slot.getPos().y===0){
                        this.onSlotScrollEnd(slot);
                    }
                }));
            }
        },
        //slot当前slot,item当前滚槽的所有对象，index第几个滚槽
        initSlot:function(slot,items,value,preventNext){
            //依据数据初始化滚轮内容
			slot.preventNext = preventNext;
            slot.dataItem = items;
            slot.params.items = slot.items = array.map(items, function(item) {
                return [item._id, item.label];
            });
            slot.adjust();
            slot.domNode.innerHTML = "";
            slot.buildRendering();
            slot.init();
            slot.resize();
            
            //设置当前值
            this.slotValues[slot.slotIndex] = value;
            var valueLabel = "",key = this._getDataKey(value);
            array.some(items,function(item){
                if(key===this._getDataKey(item)){
                    valueLabel = item.label;
                    this.slotValues[slot.slotIndex] = item;
                    return true;
                }
            },this);
            
            //设置滚轮的当前选中项
            slot.set("value",valueLabel);
        },
        
        //完成选择
        selectFinished:function(){
            //隐藏overlay
            //返回当前选中的数据
            overlay.hide();
            this._selectDef.resolve(this.getSelectedValue());
        },
        //取消选择
        selectCanceled:function(){
            //隐藏overlay
            //返回修改之前的数据
            overlay.hide();
            this._selectDef.resolve(this.getOriginValue());
        },
        //返回数据时，全部应该返回null
        getTargetData:function(data){
            return "object"===typeof(data)&&data&&(!data._id)?null:data;
        },
        
        
        //以下接口需要重写，直接通过实例化本对象来实现功能
        //初始化原始数据。slotValues等
        initSelect:function(){
        },
        //当slot依据初始值初始化完成之后的事件
        onSlotInited:function(slot){
            
        },
        //获取编辑之后的数据
        getSelectedValue:function(){
        },
        //获取编辑之前的数据
        getOriginValue:function(){
        },
        //获取第一列的数据
        loadRootData:function(){
        },
        //依据上级ID加载数据
        loadDataByUpper:function(upId){
        }
    });
});