﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dijit/registry",
    "dojo/query",
    "dojo/dom-attr",
    "dojo/dom-class",
    "cq/biz/ts/order",
    "cq/biz/user/user",
    "cq/util/sys/date"
], function(declare, lang, array, registry, query, domAttr, domClass, order, user, date){
    return declare("cq.ui.w.t.OrderList", null, {
        indexOrderListElement:function(){
            this.order_list = registry.byNode(query(".order_list",this.domNode)[0]);
            this.orderHeader = query(".orderHeader",this.domNode)[0];
        },
        initOrderListEvent:function(){
            this.connect(this.orderHeader, "onclick", this.orderHeaderClick);
            this.connect(this.order_list,"onDefRendered",function(data){
                var items = data.data.items;
                array.forEach(items,function(item){
                    // 转换订单日期
                    item.lastMod = item.lastMod?date.formatISODate(item.lastMod):"";
                    // 获取订单状态
                    item.state = "";
                    item.btnClass = "";
                    item.btnLabel = "";
                    if(item.isPayed==false&&item.isCanceled==false){
                        item.state = "待付款";
                        item.btnClass = "toCancel";
                        item.btnLabel = "取消订单";
                    }else if(item.isPayed==false&&item.isCanceled){
                        item.state = "已取消";
                    }else if(item.isPayed&&item.isCanceled){
                        item.state = "已退款";
                    }else if(item.isPayed&&(false===item.isAccepted||false===item.isDelived)&&false===item.isCanceled){
                        item.state = "待发货";
                    }else if(item.isPayed&&item.isAccepted&&item.isDelived&&item.isReceipted===false&&false===item.isCanceled){
                        item.state = "待收货";
                        item.btnClass = "toReceipt";
                        item.btnLabel = "确认收货";
                    }else if(item.isPayed&&item.isReceipted&&false===item.isCanceled&&false===item.isClosed){
                        item.state = "待评价";
                    }else if(item.isPayed&&item.isClosed&&false===item.isCanceled){
                        item.state = "已完成"
                    }
                });
            });
            this.connect(this.order_list.domNode,"onclick",function(e){
                var dom = e.target;
                if(domClass.contains(dom,"orderState")){
                    var orderId = domAttr.get(dom,"orderId");
                    if(domClass.contains(dom,"toCancel")){
                        order.onCancelAll(orderId).then(lang.hitch(this,function(){
                            this.initData();
                        }));
                    }else if(domClass.contains(dom,"toReceipt")){
                        order.setReceipteded(orderId).then(lang.hitch(this,function(){
                            this.initData();
                        }))
                    }else{
                        return;
                    }
                }
            });
            if(!this.pageSelectEvent){
                this.pageSelectEvent = this.connect(query(".pager_page",this.domNode)[0],"onclick",function(e){
                    var dom=e.target;
                    if(dom.tagName=="SPAN"){
                        if (e && e.target && -1 < domAttr.get(e.target, "name")) {
                            var pageIndex = parseInt(domAttr.get(e.target, "name"));
                            if (pageIndex > -1) {
                                this.refreshOrderList(pageIndex);
                            }
                        }
                    }
                });
            }
        },
        orderHeaderClick:function(e){
            var dom = e.target;
            if(dom.tagName=="LI"){
                var name = domAttr.get(dom,"name"),params = {};
                if(name=="all"){
                }
                if(name=="nopay"){
                    params.isPayed = false;
                    params.isCanceled = false;
                }
                if(name=="toDeliver"){
                    params.isPayed = true;
                    params.isDelived = false;
                    params.isCanceled = false;
                    params.isAccepted = false;
                }
                if(name=="toReceipt"){
                    params.isPayed = true;
                    params.isDelived = true;
                    params.isCanceled = false;
                    params.isAccepted = true;
                    params.isReceipted = false;
                }
                if(name=="toEva"){
                    params.isPayed = true;
                    params.isDelived = true;
                    params.isCanceled = false;
                    params.isAccepted = true;
                    params.isReceipted = true;
                    params.isClosed = false;
                }
                // 清空其他节点的选中效果
                if(query(".select",this.orderHeader)[0]) domClass.remove(query(".select", this.orderHeader)[0], "select");
                // 添加当前选中节点的选中效果
                domClass.add(dom, "select");
                // 刷新数据
                this.order_list.query = params;
                this.refreshOrderList();
            }
        },
        refreshAllList:function(){
            user.hasLogin().then(lang.hitch(this,function(s){
                if(s){
                    this.refreshOrderList();
                }else{
                    user.nextPage = "orderList";
                    window.moveToView("login");
                }
            }));
        },
        refreshOrderList:function(pageIndex){
            this.order_list.target = order.getWithDetailsUrl();
            this.order_list.refreshList(pageIndex).then(lang.hitch(this, function (data) {
                this.order_list.onDataReady(data);
            }));
        }
    });
});