﻿define([
    "dojo/_base/declare",
    "dojo/dom-attr",
    "dijit/registry",
    "dojo/query",
    "dojo/_base/lang",
    "dojo/_base/array",
    "cq/biz/sms/sms",
    "cq/biz/user/user",
], function(declare, domAttr, registry, query, lang, array, sms, user) {
    return declare("cq.ui.w.u.ForgetPwd", null, {
        indexForgetPwdElement: function() {
            array.forEach(["PhoneNum", "VCode", "pwd", "cpwd", "getVCode", "logPwdBtn"], function(c){
                this[c] = query("." + c, this.domNode)[0];
            }, this);
        },
        initForgetPwdEvent: function(){
            // 手机号失去焦点时检查手机号是否已注册
            this.connect(this.PhoneNum, "onblur", lang.hitch(this, this.verifyNumber));
            // 获取验证码
            this.connect(this.getVCode, "onclick", lang.hitch(this, this.getVetify));
            // 确认提交
            this.connect(this.logPwdBtn, "onclick", lang.hitch(this, this.onVerify));
        },
        verifyNumber: function() {
            var verStore;
            if (/^(0|86|17951)?(13[0-9]|15[012356789]|17[678]|18[0-9]|14[57])[0-9]{8}$/.test(this.PhoneNum.value)) {
                if (verStore !== this.PhoneNum.value) {
                    user.onCheckExist(this.PhoneNum.value).then(lang.hitch(this,function(data) {
                        if (data.data.ok === 0) {
                            alert("手机号未注册");
                            this.emptyForgetPwdData();
                        }
                    }));
                }
            } else {
                this.emptyForgetPwdData();
                alert("请输入正确的11位电话号码");
            }
            verStore = this.PhoneNum.value;
        },
        emptyForgetPwdData: function(){
            this.PhoneNum.value = ""; 
            this.VCode.value = ""; 
            this.pwd.value = ""; 
            this.cpwd.value = "";
            domAttr.remove(this.PhoneNum,"readonly");
            this.getVCode.innerHTML = "获取验证码";
            this._clear();
        },
        // 提交处理
        onRegister: function() {
            user.resetPassword({
                "userName": this.PhoneNum.value,
                "password": this.cpwd.value,
                "vCode": this.VCode.value
            }).then(lang.hitch(this, function(result) {
                if (result.data.ok === 1) {
                    // 若注册成功，自动执行登录的动作
                    user.loginByPassword(this.PhoneNum.value,this.cpwd.value).then(lang.hitch(this,function(data1){
                        if(data1.data.ok === 1){
							this.emptyForgetPwdData();
							window.moveToView(user.nextPage || "main");
                        }
                    }));
                } else if (result.data.status === 101) {
                    alert("验证码错误");
                } else if (result.data.status === 102) {
                    alert("验证码已过期");
                } else if (result.data.status === 310) {
                    alert("用户名已存在");
                }
            }));
        },
        // 验证字段
        onVerify: function() {
            // 提示处理
            if (!/^(0|86|17951)?(13[0-9]|15[012356789]|17[678]|18[0-9]|14[57])[0-9]{8}$/.test(this.PhoneNum.value)) {
                alert("请输入正确的11位电话号码");
            } else if (!this.VCode.value) {
                alert("验证码不能为空");
            } else if (!this.pwd.value || !this.cpwd.value) {
                alert("密码不能为空");
            } else if (this.pwd.value !== this.cpwd.value) {
                alert("两次密码不一致");
            } else {
                this.onRegister();
            }
        },
        // 获得验证码
        getVetify: function() {
            var number = this.PhoneNum.value;
            if (/^(0|86|17951)?(13[0-9]|15[012356789]|17[678]|18[0-9]|14[57])[0-9]{8}$/.test(number)) {
                domAttr.set(this.PhoneNum,"readonly","readonly");
                if(!this.resetTime||this.resetTime<=0){
                    sms.sendVcode(number);
                    this.resetTime = 60;
                    this.timerFunction();
                }
            } else {
                alert("请输入正解的11位电话号码");
            }
        },
        timerFunction:function(){
            this.timer = setTimeout(lang.hitch(this,function(){
                this.timerFunction();
            }),1000);
            if(this.resetTime == 0){
                query(".getVCode",this.domNode)[0].innerHTML= "重新获取";
                clearTimeout(this.timer);
            }else{
                this.getVCode.innerHTML = "重新获取("+this.resetTime+")";
                this.resetTime = this.resetTime - 1;
            }
        },
        _clear: function(){
            if(this.timer){
                clearInterval(this.timer);   
            }
        }
    });
});
