﻿define([
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-construct",
    "dojo/io/script",
    "dojo/Deferred",
    "dojo/DeferredList",
    "dojo/store/JsonRest",
    "./date",
    "./userAgent",
    "../../app/widget/utils/listLayout"
],function(lang, array, domConstruct, script, Deferred, DeferredList, JsonRest, date, userAgent,listLayout){
    return lang.mixin(lang.getObject("cq.util.sys.xlsx",true),{
        //参考cqthink/dbx/getExel对象
        //将数据导出为xlsx文件
        //sheetData：object
        //    columns：[{field:"列属性名",fieldName:"列名"}]
        //    datas：[{field:value}]
        //生成xlsx文件并已指定的文件名下载
        "export":function(sheetData,fileName){
            console.log(arguments);
            this.init().then(lang.hitch(this, function(flag){
                if(!flag){
                    alert("加载xlsx失败!");
                }else{
                    // 导出为xlsx文件
                    // 生成xlsx对应格式的对象
                    // console.log(this);
                    this.initXlsxFile(sheetData, fileName);
                    this._downloadBlod(true);
                }
            }));
        },
        // 加载xlsx相关组件
        init:function(){
            var def = new Deferred(),succeed = true;
            if("undefined"!=typeof(xlsx)){
                def.resolve(succeed);
            }else{
                new DeferredList(array.map(["jszip.js","jszip-deflate.js","jszip-inflate.js","jszip-load.js","xlsx.js"],function(fname){
                    var d = new Deferred();
                    script.get({url:require.toUrl("cq/util/sys/xlsx/"+fname),load:function(text){
                        d.resolve();
                    },error:function(){
                        succeed = false;
                        d.resolve();
                    }});
                    return d;
                })).then(function(){
                    def.resolve(succeed);
                });
            }
            return def;
        },
        initXlsxFile:function(sheetData, fileName){
            this._file = {
                worksheets: [{}],
                creator: 'cqthink',
                created: new Date(),
                lastModifiedBy: 'cqthink',
                modified: new Date(),
                activeWorksheet: 0
            };
            var data = this._file.worksheets[0];
            data.name = fileName;
            data.data = [];
            data.data[0] = array.map(sheetData.columns, function(item, index){
                return item.fieldName||item.label;
            });
            data.data = data.data.concat(array.map(sheetData.datas , function(item, index){
                return array.map(sheetData.columns, function(i){
                    return item[i.field];
                });
            }));
            this._file.worksheets[0] = data;
        },
        // 下载文件，isCsv  是否下载未csv文件格式，因为下载为xlsx文件需要大量计算，影响效率，因此直接下载为csv文件
        _downloadBlod:function(isCsv){
            var node = domConstruct.create("a",{
                download:(this._file.worksheets[0].name||"导出数据")+(isCsv?".csv":".xlsx"),
                style:"display:block;margin:10px auto;padding:20px;",
                innerHTML:"确认"
            },document.body);
            if(isCsv){
                var getData = function(s){
                    if(s&&(s.value||0===s.value||""===s.value||false===s.value)){
                        if(-1<(""+s.value).indexOf(",")){
                            return '"'+s.replace(/"/g,'""')+'"';
                        }else{
                            return s.value;
                        }
                    }
                    return s;
                };
                node.href = window.webkitURL.createObjectURL(new Blob(["\ufeff"+array.map(this._file.worksheets[0].data,function(d){
                    return array.map(d,function(dd){
                        return getData(dd);
                    }).join(",");
                }).join("\n")],{type:"data:text/csv"}));
            }else{
                node.href = window.webkitURL.createObjectURL(xlsx(this._file).blob);
            }
            if(userAgent.isMobile()){
                var div = domConstruct.create("div",{
                    style:"position:absolute;bottom:20px;",
                    onclick:function(){
                        div.parentNode.removeChild(div);
                    },
                    innerHTML:"<span style='display:block;margin:10px auto;padding:20px;'> 取消 </span>"
                },document.body,"first");
                domConstruct.place(node,div,"first");
            }else{
                node.click();
                document.body.removeChild(node);
            }
        },
        //xlsx.exportData("/biz/shop/bs/district/load_by_level?level=2",{
        //},[{
        //  field:"_id",
        //  fieldName:"id"
        //},{
        //  field:"distName",
        //  fieldName:"省份"
        //}],"你好吗")
        exportData:function(url,query,fields,fileName){
            var fields =  listLayout.initLayoutFormatter(fields);
            var file = {
                columns:fields,
                datas:[]
            },store = new JsonRest({
                target:url
            }),start = 0,pageSize = 1000,
            //dfields = [],
            onTotalLoaded = function(total){
                // 判断服务器端接口是否支持分页，不支持时，返回的total为null
                store._cqTotal = total;
            },
            startQuery = lang.hitch(this,function(){
                //加载数据，加载完成之后处理数据
                var queryResult = store.query(query,{
                    start:start,count:pageSize
                });
                queryResult.total.then(onTotalLoaded);
                queryResult.then(dataLoaded);
            }),dataLoaded = lang.hitch(this,function(datas){
                //处理数据
                datas = datas.items||datas||[];
                start += pageSize;
                //处理完成之后判断是需要加载下一页，还是生成文件
                if(datas.length){
                    array.forEach(datas,function(data){
                        file.datas.push(data);
                        /*array.forEach(dfields,function(field){
                            field.dValue(data);
                        });*/
                        array.forEach(fields,function(field){
                            data[field.field] = field.formatter(data[field.field],data);
                        });
                    });
                }
                if(datas.length===pageSize&&store._cqTotal){
                    // 本页取满了且服务器端支持分页时才加载下一页
                    startQuery();
                }else{
                    this.export(file,fileName);
                }
            });
            /*array.forEach(fields,function(field){
                dfields.push(field);
                if(field.options){
                    field.dValue = this.decorateOptionValue;
                }else if(field.type==="date"){
                    field.dValue = this.decorateDateValue;
                }else if(field.type==="textarea"){
                    field.dValue = this.decorateTextValue;
                }else if(field.type==="time"){
                    field.dValue = this.decorateTimeValue;
                }else if(field.field.indexOf(".")){
                    field.dValue = this.decorateRefValue;
                }else{
                    dfields.pop();
                }
            },this);*/
            store._cqTotal = 0;
            startQuery();
        }/*,
        decorateOptionValue:function(data){
            data[this.field] = this.options[data[this.fields]];
        },
        decorateDateValue:function(data){
            data[this.field] = data[this.field]?date.formatISODate(data[this.field]):"";
        },
        decorateTextValue:function(data){
            data[this.field] = data[this.field]?(data[this.field]).replace(/\n/g,""):"";
        },
        decorateTimeValue:function(data){
            data[this.field] = data[this.field]?date.formatISODateMinutes(data[this.field]):"";
        },
        decorateRefValue:function(data){
            var fields = this.fields||this.field.split(".");
            var fieldData = data;
			fields = lang.clone(fields);
            while(fields.length&&fieldData){
                fieldData = fieldData[fields.shift()];
            }
            data[this.field] = fieldData;
        }*/
    });
});