﻿// Pager的设计思想
// 1.需求。
//         1.1.显示指定对象的总页数和当前页。
//        1.2.页码跳转。支持前、后、最前、最后、手工输入到指定页、点击当前页先后若干页。
//        1.3.支持显示配置。是否显示最前最后，显示当前页前后各N个页码的链接，是否显示总记录数，是否显示导出Excel。
//        1.4.支持绑定机制。
// 2.与被绑定的对象的耦合机制。
//        2.1.支持绑定到任意js对象，可通过id来指定要绑定的dijit元素的id，不支持直接绑定到html元素。
//        2.2.绑定关系指被绑定的对象通知Pager当前页和总页数的机制，以及点击分页需要调用的函数。
//        2.3.与cqthink.wb.List2的绑定。Pager必须放到被List2中。Pager自动初始化与List2的绑定关系。
//        2.4.TODO.与Grid的绑定。Pager由Grid创建，Grid负责创建和初始化Pager。
//        2.5.TODO.与g_d数据源绑定。需设置Pager的rs_db_path,Pager自动连接到g_d.getDataWithPage来初始化数据绑定。
//        2.6.绑定到任意JS对象。需要手工初始化绑定关系。
// 上下文错误，母浚驰更改
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/connect",
    "dojo/query",
    "dijit/_Widget",
    "dijit/_Templated",
    "dojo/dom-construct",
    "dijit/registry"
], function(declare, lang, connect, query, _Widget, _Templated, domConstruct, registry) {
    domConstruct.create("style",{
        innerHTML:".totalCount{color:#999999;}.pagePadding{padding-right:5px;}"
    },document.head,"first");
    return declare("cq.widgets.DijitPager", [_Widget, _Templated], {
        showFirstLast: "none", // 显示最前最后
        showPages: 10, // 前后最多显示10页
        showTotal: "",
        showExcel: "none",
        forList: "",
        page: 1,
        pageCount: 1,
        totalCount: 1,
        id: "",
        templateString: '<div id="${id}" ><span><a href="javascript:void(0);" dojoAttachEvent="onclick:onFirstPage" style="display:${showFirstLast};">最前</a></span><span class="pager_page"  > </span><span><a href="javascript:void(0);" style="display:${showFirstLast};" dojoAttachEvent="onclick:onLastPage">最后</a></span></div>',
        postMixInProperties: function() {
            this.inherited(arguments);
            // 初始化与列表的绑定关系。
            if (this.forList) {
                var list = registry.byId(this.forList);
                if (!list) return;
                connect.connect(list, "onRendered", this, this.printHtml);
            }
        },
        onFirstPage: function() {},
        onLastPage: function() {},
        onExcel: function() {},
        connectEvent: function() {
            connect.connect(g_d, "getDataWithPage", lang.hitch(this, this.printHtml));
            if (this.inGrid){
                connect.connect(this.inGrid, "_onFetchComplete", lang.hitch(this, this.printHtml));
            }
        },
        getPage: function() { //丛url获得当前页数,如果变量重复只获取最后一个
            var list = registry.byId(this.forList);
            if (!list){
                return;
            }
            this.page = list.curPage;
            this.pageCount = list.pageCount;
            this.totalCount = parseInt(list.totalCount, 10);
        },
        checkPages: function() { //进行当前页数和总页数的验证
            if (isNaN(parseInt(this.page, 10))){
                this.page = 1;
            }
            if (isNaN(parseInt(this.pageCount, 10))){
                this.pageCount = 1;
            }
            if (this.page < 1){
                this.page = 1;
            }
            if (this.pageCount < 1){
                this.pageCount = 1;
            }
            if (this.page > this.pageCount){
                this.page = this.pageCount;
            }
            this.page = parseInt(this.page, 10);
            this.pageCount = parseInt(this.pageCount, 10);
        },
        createHtml: function(mode) { //生成html代码
            var strHtml = '',
                prevPage = this.page - 1,
                nextPage = this.page + 1,
                endPage;
            if (mode == '' || typeof(mode) == 'undefined'){
                mode = 0;
            }
            // node-->mode
            if(2===mode){
                //模式2 (前后缩略,页数,首页,前页,后页,尾页)
                if (prevPage < 1) {
                    strHtml += '<span>前页</span>';
                } else {
                    strHtml += '<span class="pagePadding"><a href="javascript:dijit.registry.byId(\'' + this.forList + '\').refreshList(' + prevPage + ');">前页</a></span>';
                }
                if (this.page != 1) strHtml += '<span class="pagePadding"><a href="javascript:dijit.registry.byId(\'' + this.forList + '\').refreshList(1);">1</a></span>';
                if (this.page >= 5) strHtml += '<span class="pagePadding">...</span>';
                if (this.pageCount > this.page + 2) {
                    endPage = this.page + 2;
                } else {
                    endPage = this.pageCount;
                }
                for (var i = this.page - 2; i <= endPage; i++) {
                    if (i > 0) {
                        if (i == this.page) {
                            strHtml += '<span class="pagePadding currentPage">' + i + '</span>';
                        } else {
                            if (i != 1 && i != this.pageCount) {
                                strHtml += '<span  class="pagePadding"><a href="javascript:dijit.registry.byId(\'' + this.forList + '\').refreshList(' + i + ');" >' + i + '</a></span>';
                            }
                        }
                    }
                }
                if (this.page + 3 < this.pageCount) strHtml += '<span class="pagePadding">...</span>';
                if (this.page != this.pageCount) strHtml += '<span class="pagePadding"><a href="javascript:dijit.registry.byId(\'' + this.forList + '\').refreshList(' + this.pageCount + ');">' + this.pageCount + '</a></span>';
                if (nextPage > this.pageCount) {
                    strHtml += '<span class="pagePadding">后页</span>';
                } else {
                    strHtml += '<span class="pagePadding"><a href="javascript:dijit.registry.byId(\'' + this.forList + '\').refreshList(' + nextPage + ');">后页</a></span>';
                }
                strHtml += '<span class="totalCount totalCountRecord" style="display:' + this.showTotal + ';">总' + this.totalCount + '条记录</span>';
            }
            return strHtml;
        },
        printHtml: function() {
            this.getPage();
            this.checkPages();
            this.showTimes += 1;
            this.buildRendering();
            query(".pager_page", this.domNode.id)[0].innerHTML = this.createHtml(2);
        }
    });
});