define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojox/mobile/parser",
    "dijit/_Widget",
    "dojox/dtl/_base",
    "dojox/dtl/filter/strings"
],function(declare,lang,domConstruct,domStyle,parser,Widget,dtl,strings){
    lang.mixin(strings,{
        floatformat: function(value, arg){
            arg = parseInt(arg || -1, 10);
            if(isNaN(value)||""==value||null==value||undefined==value) return "";
            value = parseFloat(value);
            var m = value - value.toFixed(0);
            if(!m && arg < 0){
                return value.toFixed();
            }
            value = value.toFixed(Math.abs(arg));
            return (arg < 0) ? parseFloat(value) + "" : value;
        }
    });
    var integers = lang.mixin(lang.getObject("cqthink.dtl.filter.integers",true),{
        cAmount: function (currencyDigits) {
            // Constants:
            var MAXIMUM_NUMBER = 99999999999.99;
            // Predefine the radix characters and currency symbols for output:
            var CN_ZERO = "零";
            var CN_ONE = "壹";
            var CN_TWO = "贰";
            var CN_THREE = "叁";
            var CN_FOUR = "肆";
            var CN_FIVE = "伍";
            var CN_SIX = "陆";
            var CN_SEVEN = "柒";
            var CN_EIGHT = "捌";
            var CN_NINE = "玖";
            var CN_TEN = "拾";
            var CN_HUNDRED = "佰";
            var CN_THOUSAND = "仟";
            var CN_TEN_THOUSAND = "万";
            var CN_HUNDRED_MILLION = "亿";
            //var CN_SYMBOL = "人民币";
            var CN_SYMBOL = "";
            var CN_DOLLAR = "元";
            var CN_TEN_CENT = "角";
            var CN_CENT = "分";
            var CN_INTEGER = "整";
            // Variables:
            var integral; // Represent integral part of digit number.
            var decimal; // Represent decimal part of digit number.
            var outputCharacters; // The output result.
            var parts;
            var digits, radices, bigRadices, decimals;
            var zeroCount;
            var i, p, d;
            var quotient, modulus;
            // Validate input string:
            currencyDigits = currencyDigits.toString();
            if (currencyDigits == "") {
                //alert("Empty input!");
                return "";
            }
            if (currencyDigits.match(/[^,.\d]/) != null) {
                //alert("Invalid characters in the input string!");
                return "";
            }
            if ((currencyDigits).match(/^((\d{1,3}(,\d{3})*(.((\d{3},)*\d{1,3}))?)|(\d+(.\d+)?))$/) == null) {
                //alert("Illegal format of digit number!");
                return "";
            }
            // Normalize the format of input digits:
            currencyDigits = currencyDigits.replace(/,/g, ""); // Remove comma delimiters.
            currencyDigits = currencyDigits.replace(/^0+/, ""); // Trim zeros at the beginning.
            // Assert the number is not greater than the maximum number.
            if (Number(currencyDigits) > MAXIMUM_NUMBER) {
                alert("您输入的金额太大，请重新输入!");
                return "";
            }
            // Process the coversion from currency digits to characters:
            // Separate integral and decimal parts before processing coversion:
            parts = currencyDigits.split(".");
            if (parts.length > 1) {
                integral = parts[0];
                decimal = parts[1];
                // Cut down redundant decimal digits that are after the second.
                decimal = decimal.substr(0, 2);
            } else {
                integral = parts[0];
                decimal = "";
            }
            // Prepare the characters corresponding to the digits:
            digits = new Array(CN_ZERO, CN_ONE, CN_TWO, CN_THREE, CN_FOUR, CN_FIVE, CN_SIX, CN_SEVEN, CN_EIGHT, CN_NINE);
            radices = new Array("", CN_TEN, CN_HUNDRED, CN_THOUSAND);
            bigRadices = new Array("", CN_TEN_THOUSAND, CN_HUNDRED_MILLION);
            decimals = new Array(CN_TEN_CENT, CN_CENT);
            // Start processing:
            outputCharacters = "";
            // Process integral part if it is larger than 0:
            if (Number(integral) > 0) {
                zeroCount = 0;
                for (i = 0; i < integral.length; i++) {
                    p = integral.length - i - 1;
                    d = integral.substr(i, 1);
                    quotient = p / 4;
                    modulus = p % 4;
                    if (d == "0") {
                        zeroCount++;
                    } else {
                        if (zeroCount > 0) {
                            outputCharacters += digits[0];
                        }
                        zeroCount = 0;
                        outputCharacters += digits[Number(d)] + radices[modulus];
                    }
                    if (modulus == 0 && zeroCount < 4) {
                        outputCharacters += bigRadices[quotient];
                    }
                }
                outputCharacters += CN_DOLLAR;
            }
            // Process decimal part if there is:
            if (decimal != "") {
                for (i = 0; i < decimal.length; i++) {
                    d = decimal.substr(i, 1);
                    if (d != "0") {
                        outputCharacters += digits[Number(d)] + decimals[i];
                    }
                }
            }
            // Confirm and return the final output string:
            if (outputCharacters == "") {
                outputCharacters = CN_ZERO + CN_DOLLAR;
            }
            if (decimal == "") {
                outputCharacters += CN_INTEGER;
            }
            outputCharacters = CN_SYMBOL + outputCharacters;
            return outputCharacters;
        }
    });
    dtl.register.filters("cqthink.dtl.filter", {
        "integers": ["cAmount"]
    });
    return declare("cq.widgets.Template",Widget,{
        _dijitTemplateCompat: true,
        id: "",
        //ZCH使这个控件在新IDE中可用
        isDesign: (window.location.pathname == "/dev/"),
        all_window: {},
        _template: null,
        templateString:"",
        rs_wb_wait: "",
        postMixInProperties: function () {
            this.inherited(arguments);
            this.all_window = window;
            if (!this.isDesign) {
                // 运行模式下，将内容放置于模板字符串中，并清空内容
                // 由refreshList调用来显示内容
                var h = this.templateString||this.srcNodeRef.innerHTML;
                this._templateString = h.replace(/%25/g, "%").replace(/%7B/g, "{").replace(/%7D/g, "}").replace(/%20/g, ' ');
                this.srcNodeRef.innerHTML = "";
                //赵春辉    List乱码问题    2015-10-22
                setTimeout(lang.hitch(this,function(){
                    //控制不透明度
                    var node = this.domNode||this.srcNodeRef;
                    if(node){
                        domStyle.set(node,"opacity",1);
                    }
                }),100);
            }
        },
        refreshList: function () {
            if (!this._template) this._template = new dtl.Template(this._templateString);
            var context = new dtl._Context(this);
            this.destroyDescendants(false);
            this.domNode.innerHTML = this.onListHtmlReady(this._template.render(context));
            //控制不透明度
            domStyle.set(this.domNode,"opacity",1);
            parser.parse(this.domNode);
            this.onRendered();
        },
        onListHtmlReady : function(html) {
            var node = domConstruct.create("div",{
                innerHTML:html
            }),dealNode = function(node){
                if(node.nodeType!=3){
                    var i=node.childNodes.length;
                    while(i--){
                        dealNode(node.childNodes[i]);
                    }
                }else{
                    node.data = node.data.replace(/(^\s+)|(\s+$)/g,"");
                }
            };
            dealNode(node);
            
            return node.innerHTML;
        },
        buildRendering: function () {
            this.inherited(arguments);
            if (!this.containerNode) {
                this.containerNode = this.domNode;
            }
            if(!this.isDesign && !this.rs_wb_wait) this.refreshList();
        },
        onRendered: function (data) {}
    });
});