﻿define([
    "dojo/_base/lang",
    "dojo/dom",
    "dojo/dom-attr",
    "dojo/dom-style"
],function(lang,dom,domAttr,domStyle){
    var animation = lang.getObject("cq.widgets.animation", true);
    var easings = {
        linear: function(initVal, diff, progress, escapedTime) {
            return initVal + diff * progress;
        },
        swing: function(initVal, diff, progress, escapedTime) {
            return ( ( -Math.cos( progress * Math.PI ) / 2 ) + 0.5 ) * diff + initVal;
        }
    };
    var ID_ATTR_NAME = '_jRaiserNodeId_';
    lang.mixin(animation,{
        rNumber:/^[+-]?\d+(?:\.\d+)?[^\s]*$/,
        rRelNumber : /^([+-])?(\d+(?:\.\d+)?)[^\s]*$/,
        rColor : /color$/i,
        rSharpColor : /^#[a-f0-9]{6}$/i,
        rRGBColor : /^rgb\((\d+),\s(\d+),\s(\d+)\)$/,
        rScroll : /^scroll/,
        defaultStyleValues :{ opacity: 1 },
        animationQueue :[ ],    // 动画队列
        animationTimerId:"",// 执行动画的setInterval返回的timer id
        animationSpace:{},
        autoId:0,
        uniqueId: function(node) {
            var uid = node[ID_ATTR_NAME];
            if (uid === undefined ) {
                uid = node[ID_ATTR_NAME] = new Number(++this.autoId);
            }
            return uid ? uid.valueOf() : uid;
        },
        // 转换样式值，数值去掉单位
        parseStyleValue:function(val) {
            if (typeof val === 'string') {
                if (this.rNumber.test(val) ) {
                    val = parseFloat(val, 10) || 0;
                } else if ( this.rSharpColor.test(val) ) {
                    // #开头的16进制颜色值转10进制
                    val = [
                        parseInt(val.substr(1, 2), 16),
                        parseInt(val.substr(3, 2), 16),
                        parseInt(val.substr(5, 2), 16)
                    ];
                } else if ( this.rRGBColor.test(val) ) {
                    // rgb开头的10进制颜色值转为数字
                    val = [
                        parseInt(RegExp.$1, 10),
                        parseInt(RegExp.$2, 10),
                        parseInt(RegExp.$3, 10)
                    ];
                } else {
                    val = val.toLowerCase();
                }
            }
            return val;
        },
        // 获取与最终样式相对应的初始样式值
        getCurStyles:function(node, finalStyles) {
            var curStyles = { }, val="";
            for (var n in finalStyles) {
                if (n === 'width' || n === 'height') {
                    //val = $style.getSize(node, n);
                } else if ( this.rScroll.test(n) ) {
                    //val = $style.getScroll( node, n.replace(rScroll, '') );
                } else {
                    val = this.parseStyleValue( domStyle.get(node, n));
                }
                if ( val === '' && this.defaultStyleValues.hasOwnProperty(n) ) {
                    val = this.defaultStyleValues[n];
                }
                curStyles[n] = val;
            }
            return curStyles;
        },
        //执行动画
        animate:function(node,finalStyles, options) {
            for (var n in finalStyles) {
                finalStyles[n] = this.parseStyleValue(finalStyles[n]);
            }
            //node.forEach(function(n) {
            this.startAnimation(node, finalStyles, options, true);
            //});
            return node;
        },
        //停止执行动画
        stop:function(node){
            //node.forEach(function(n) {
            this.stopAnimation(node);
            //});
            return node;
        },
        //停止动画
        stopAnimation:function(node){
            var actionId = this.animationSpace[this.uniqueId(node)];
            if (actionId != null) {
                this.clearAnimation(actionId);
                delete this.animationSpace[this.uniqueId(node)];
            }
        },
        //清除动画
        clearAnimation:function(id) {
            this.animationQueue[id] = null;
            // 没有动画，可以清理动画队列
            if (this.animationQueue.join('') === '') {
                this.animationQueue = [];
                if (this.animationTimerId) {
                    clearInterval(this.animationTimerId);
                    this.animationTimerId = null;
                }
            }
        },
        //执行动画
        doAnimation:function(action) {
            var actionId = this.animationQueue.push(action) - 1;
            if (!this.animationTimerId) {
                this.animationTimerId = setInterval(lang.hitch(this,function() {
                    for (var i = 0, len = this.animationQueue.length; i < len; i++) {
                        this.animationQueue[i] && this.animationQueue[i]();
                    }
                }), 13);
            }
            // 返回在队列中的序号
            return actionId;
        },
        // 计算动画过程中的下一个值
        calculateVal:function(cVal, fVal, easing, progress) {
            var nVal;
            if (typeof cVal === 'number') {
                // 相对数值，如 '+5'
                if ( typeof fVal === 'string' && this.rRelNumber.test(fVal) ) {
                    fVal = cVal + parseFloat(fVal, 10);
                }
                if (typeof fVal === 'number') {
                    nVal = cVal + (fVal - cVal) * easing(0, 1, progress);
                    // 防止超出界限值
                    if ( (fVal > cVal && nVal > fVal) || (fVal < cVal && nVal < fVal) ) {
                        nVal = fVal;
                    }
                }
            } else if (lang.isArray(cVal) && lang.isArray(fVal) ) {
                nVal = [ ];
                cVal.forEach(function(v, i) {
                    nVal[i] = parseInt(this.calculateVal(v, fVal[i], easing, progress), 10);
                });
            }
            return nVal || fVal;
        },
        /**
            * 对指定节点执行动画
            * @method start
            * @param {Element} node 节点
            * @param {Object} finalStyles 最终样式
            * @param {Object} [options] 其他参数
            * @param {Number} [options.duration=400] 动画时长
            * @param {Function|String} [options.easing='linear'] 动画过渡效果
            * @param {Function} [options.callback] 动画结束后的回调
        */
        // isPass参数主要用于循环调用的时候，忽略对最终样式值的修正
        startAnimation:function(node, finalStyles, options, isPass) {
            options = lang.mixin(
                options,{
                duration: 400,
                easing: 'linear'
            });
            var easing = options.easing;
            if (typeof easing === 'string') {
                easing = easings[easing];
            }
            if (!easing) {
                throw new Error('please specify easing');
            }
            if (!isPass) {
                // 修正最终样式的样式值
                for (var n in finalStyles) {
                    finalStyles[n] = this.parseStyleValue(finalStyles[n]);
                }
            }
            // 获取节点的当前样式
            var curStyles = this.getCurStyles(node, finalStyles);
            // 停止已有的动画，防止冲突
            this.stopAnimation(node);
            // 先设为可见，不然看不到动画效果
            if (finalStyles.visibility === 'visible') {
                domStyle.set(node, 'visibility', 'visible');
            }
            if (finalStyles.display === 'block') {
                domStyle.set(node, 'display', 'block');
            }
            var nodeId = this.uniqueId(node), startTime = + new Date();
            var actionId = this.doAnimation(lang.hitch(this,function() {
                // 计算动画进度
                var escapedTime = new Date() - startTime, progress = escapedTime / options.duration;
                var cVal,    // 当前样式值
                fVal,    // 最终样式值
                nVal;    // 新样式值
                for (var n in finalStyles) {
                    if (n === 'visibility' || n === 'display') { continue; }
                    cVal = curStyles[n];
                    fVal = finalStyles[n];
                    if (cVal != fVal) {
                        nVal = this.calculateVal(cVal, fVal, easing, progress);
                        if ( this.rScroll.test(n) ) {
                            //$style.setScroll(node, n.replace(rScroll, ''), nVal);
                        } else {
                            if(n=="left") domStyle.set(node, n,nVal+"px");
                            else domStyle.set(node, n,
                            this.rColor.test(n) ? 'rgb(' + nVal.join(', ') + ')' : nVal);
                        }
                    }
                }
                if (escapedTime >= options.duration) {
                    this.stopAnimation(node);
                    // 确保最终样式正确
                    for (var s in finalStyles) {
                        if(s=="left")  domStyle.set(node,s,finalStyles[s]+"px");
                        else domStyle.set(node,s,this.rColor.test(s) ? 'rgb(' + finalStyles[s] + ')' : finalStyles[s]);
                    }
                    // 回调
                    options.callback && options.callback.call(node);
                }
            }));
            // 记录动画的Id，方便清除
            this.animationSpace[nodeId] = actionId;
        }
    });
    return animation;
});