// 创建人:赵春辉
// 创建日期:2016-06-03
// 用途:酉阳网站数据源支持分页的列表
define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-attr",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/store/JsonRest",
    "dojox/dtl/_base",
    "../Template"
],function(declare, lang, domAttr, domConstruct, domStyle, JsonRest, dtl, Template) {
    return declare("cq.widgets.list.RestList", Template, {
        // 分页大小
        pageSize : 20,
        // 当前显示的是第几页
        currentPage : 0,
        // 读取数据的目标地址
        target : "",
        // 附加的查询条件
        query : null,
        // 启用分页
        enablePager : false,
        // 初始化界面元素
        buildRendering : function() {
            this.inherited(arguments);
            if (!this.containerNode) {
                this.containerNode = this.domNode;
            }
            if (!this.tempNode) {
                this.tempNode = this.domNode;
            }
            if (!this.pagerNode) {
                this.pagerNode = domConstruct.create("div", {
                    "class" : "pager_page",
                    onclick : lang.hitch(this, this.onPagerClick)
                });
            }
        },
        startup : function() {
            this.inherited(arguments);
            setTimeout(lang.hitch(this, function() {
                //控制不透明度
                domStyle.set(this.domNode, "opacity", 1);
            }), 100);
        },
        refreshList : function(page) {
            if (!this.store) {
                if (!this.target) {
                    console.error("no target rest list:" + this.get("id"));
                    return;
                }
                this.store = new JsonRest({target : this.target});
            }
            this.store.target = this.target||this.store.target;
            this.currentPage = page || 0;
            this.pageSize = this.pageSize > 0 ? this.pageSize : 20;
            var queryResult = this.store.query(this.getQuery(), {
                start : this.currentPage * this.pageSize,
                count : this.pageSize
            });
            queryResult.then(lang.hitch(this, function (data) {
                this.onDataReady(data);
            }));
            queryResult.total.then(lang.hitch(this, this.onTotalLoaded));
            return queryResult;
        },
        getQuery : function() {
            if (this.query && "string" == typeof(this.query)) {
                try {
                    this.query = eval(this.query);
                } catch (e) {
                    console.error(e);
                }
            }
            return this.query || {};
        },
        setQuery : function(query) {
            this.query = query;
            return this.refreshList(0);
        },
        onTotalLoaded : function(totalCount) {
            if (this.enablePager) {
                var html = "";
                if (0 === this.currentPage) {
                    html += "";
                } else if (1 === this.currentPage) {
                    html += "<span class='pager_item' name='0'>1</span>";
                } else {
                    if (2 < this.currentPage) {
                        html += "<span class='pager_item' name='0'>首页</span>";
                        if (3 < this.currentPage) {
                            html += "<span class='pager_more'>...</span>";
                        }
                    }
                    html += "<span class='pager_item' name='" + (this.currentPage - 2);
                    html += "'>" + (this.currentPage - 1) + "</span>";
                    html += "<span class='pager_item' name='" + (this.currentPage - 1);
                    html += "'>" + this.currentPage + "</span>";
                }
                html += "<span class='pager_item selected'>";
                html += (this.currentPage + 1) + "</span>";
                var total = parseInt(totalCount / this.pageSize);
                total += (totalCount % this.pageSize ? 1 : 0) - 1;
                if (0 < total - this.currentPage) {
                    html += "<span class='pager_item' name='" + (this.currentPage + 1)
                    html += "'>" + (this.currentPage + 2) + "</span>";
                }
                if (1 < total - this.currentPage) {
                    html += "<span class='pager_item' name='" + (this.currentPage + 2)
                    html += "'>" + (this.currentPage + 3) + "</span>";
                    if (2 < total - this.currentPage) {
                        if (3 < total - this.currentPage) {
                            html += "<span class='pager_more'>...</span>";
                        }
                        html += "<span class='pager_item' name='" + total + "'>尾页</span>";
                    }
                }
                this.pagerNode.innerHTML = html;
                this.containerNode.appendChild(this.pagerNode);
            }
        },
        onPagerClick : function(e) {
            if (e && e.target && -1 < domAttr.get(e.target, "name")) {
                var pageIndex = parseInt(domAttr.get(e.target, "name"));
                if (pageIndex > -1) {
                    this.refreshList(pageIndex);
                }
            }
        },
        onDefRendered : function(data) {},
        onDataReady : function(data) {
            if (data.items) {
                data = {data : data};
            }
            this.items = data.data.items || [];
            this.data = data;
            this.onDefRendered(data);
            if (!this._template){
                this._template = new dtl.Template(this._templateString);
            }
            var context = new dtl._Context(this);
            this.beforeGetContent();
            this.tempNode.innerHTML =
            this.onListHtmlReady(this._template.render(context));
            dojo.parser.parse(this.tempNode);
            this.afterGetContent();
            //控制不透明度
            domStyle.set(this.domNode, "opacity", 1);
        },
        beforeGetContent : function() {
            if (this.pagerNode && this.pagerNode.parentNode) {
                this.pagerNode.parentNode.removeChild(this.pagerNode);
            }
            this.destroyDescendants(false);
            this.containerNode.innerHTML = "";
        },
        afterGetContent : function() {
            if (this.enablePager && this.pagerNode) {
                this.containerNode.appendChild(this.pagerNode);
            }
        }
    });
    return List2;
});