// 变更人：宋宇
// 变更时间：2016-06-13
// 变更事项：初稿
define([
    "dojo/_base/declare", // 声明对象的
    "dojo/_base/lang", // 语言工具(改变语言上下文)
    "dojox/mobile/View", // view的基类
    "dojox/mobile/EdgeToEdgeStoreList", // 用于显示数据的列表
    "cq/ui/view/_ViewMixin", // 自己封装的公共的基类
    "dojo/query", // 前台ui元素的检索
    "dijit/registry", // 去找界面元素的注册表
    "cq/biz/im/channel", // 服务器接口的代理层，方便去调服务器端的程序
    "dojo/store/JsonRest", // 从服务器加载数据的数据源
    "dojo/dom-construct",
    "dojox/mobile/ListItem",
    "dojo/dom-class"
], function (declare, lang, View, EdgeToEdgeStoreList, _ViewMixin, query, registry, channel, JsonRest, domConstruct, ListItem, domClass) {
    // module:
    //		whz/view/ChannelList
    return declare("widgets.mobile.ChannelList", [View, _ViewMixin], {
        // 取数据时的开始下标，默认为0
        start : 0,
        // 每次加载的数据条数，默认为10
        pageSize : 15,
        // 每次进入页面，重新加载数据
        onAfterShow : function () {
            if (this.storeList.store) {
                this.setQuery(this.storeList.get("query"), {
                    start : 0,
                    count : this.pageSize
                });
            } else {
                this.storeList.setStore(this.store, {}, {
                    start : 0,
                    count : this.pageSize
                });
            }
        },
        // 第一次进入页面，加载页面显示的组件
        onFirstShow : function () {
            // 页面的头部
            var option = {
                left : {
                    iconButton : ' ',
                    arrow : "left",
                    label : "返回",
                    onClick : function () {
                        history.back(); //返回浏览记录的上一个页面
                    }
                },
                title : "直播频道"
            };
            this.addHeader(option);

            // 搜索框
            option = {
                placeHolder : "请输入要查询条件，按回车查询",
                type : "search",
                class : "cqSearchBox",
                // 搜索框固定到顶部
                isHeader: true,
                onInput : lang.hitch(this, function (evt) {
                    if (evt.keyCode == 13) {
                        this.search = evt.target.value;
                        var condition = this.storeList.get("query");
                        condition.title = evt.target.value;
                        this.setQuery(condition);
                        evt.target.value = "";
                    }
                })
            };
            this.addTextBox(option);
            // 频道的显示列表
            this.storeList = new EdgeToEdgeStoreList({
                append : true,
                constraint : false,
                query : {},
                class : "icon-list-item",
                variableHeight : true,
                onComplete : lang.hitch(this, function (data) {
                    for (var i = 0; i < data.length; i++) {
                        data[i]._label = "<div class='mat-list-mname mat-list-margin'>" + data[i].title + "</div>";
                        data[i].moveTo = "#";
                        data[i]._logo = data[i].logo ? data[i].logo : 'images/userIcon.png';
                    }
                    this.storeList.generateList(data);
                    this.end = this.start + data.length - 1;
                    this.busy = false;
                }),
                queryOptions : {
                    count : this.pageSize,
                    start : this.start
                },
                itemMap : {
                    _label : "label",
                    _logo : "icon",
                    moveTo : "moveTo"
                }
            });
            // 创建数据源
            this.params = {};
            this.store = new JsonRest({
                target : channel.getLoadUrl()
            }); // 告诉在哪里获取数据源
            this.storeList.placeAt(this.centerPanel.containerNode, "last"); // 将列表放到哪里去
            this.storeList.on("click", lang.hitch(this, this.onClickList));
            this.centerPanel.on("afterscroll", lang.hitch(this, this.onLoadMore));
            this.onAfterShow();
        },
        // 列表项的点击响应事件
        onClickList : function (evt) {
            // 得到点击的ListItem
            var dom = evt.target;
            var li = registry.getEnclosingWidget(evt.target);
            if (li.declaredClass != "dojox.mobile.ListItem"){
                return;
            }
            var _id = li.params._id;
            window.moveToView("channelDetail", _id);
        },

        // 重新赋值列表并刷新显示
        setQuery : function (query) {
            this.start = 0;
            this.end = -1;
            this.storeList.destroyDescendants();
            this.storeList.setQuery(query, {
                start : this.start,
                count : 10
            });
            this.centerPanel.resize();
        },

        // 拖动加载更多
        onLoadMore : function (evt) {
            if (!this.busy && evt.afterBottom) {
                this.start = this.start + this.pageSize;
                if (this.canLoadMore(this.start)) {
                    this.busy = true;
                    console.log("加载更多频道");
                    setTimeout(lang.hitch(this, function() {
                        this.onLoadListData(this.storeList.setQuery(this.storeList.get("query"), {
                            start: this.start,
                            count: this.pageSize
                        }), this.start, this.pageSize);
                    }), 1000);
                }
            }
        }
    });
});
